# -*- coding: utf-8 -*-
from django.core.management.base import BaseCommand, CommandError
from django.db import transaction
from django.utils import timezone
from annotation.models import Text, Excerpt, Segment, Workdetail, Reject_status, Work_status
from annotation.views import checkExcerpts
from django.contrib.auth.models import User
import os, io
from lxml import etree
import json
import time
from annotation.anot2_config import NO_INTERP, SORT_ORDER

namespaces = {
  'tei': 'http://www.tei-c.org/ns/1.0',
  'xi': 'http://www.w3.org/2001/XInclude',
  'nkjp': 'http://www.nkjp.pl/ns/1.0',
  'xml': 'http://www.w3.org/XML/1998/namespace'
}

TAGER_USER = None # sprawdzany na początku

'''Komenda na dodawanie wersji ujednoznacznionej przez tager.
Próbki "czyste" muszą już być w bazie, a wersja tagera musi
być zrobiona na oryginalnych wersjach po amebie (z zachowaniem 
identyfikatorów) - czyli ann_segmentation musi być identyczny,
a w ann_morphosyntax powinny być wyłącznie dodane fragmenty
'disamb'.
'''

class Command(BaseCommand):
  help = 'Adds result of tagger work as one of annotator.'
  missing_args_message = 'Please provide "texts_dir" argument'
  
  def add_arguments(self, parser):
    # Positional argument
    parser.add_argument('texts_dir', nargs='?')


  def handle(self, *args, **options):
    texts_dir = unicode(options.get('texts_dir')) #None, jeśli texts_dir nie był podany argument
      
    self.stdout.write('Processing texts, texts_dir: %s' % texts_dir)
    res = self.__load(texts_dir)
    if not res:
      self.stdout.write('Successfully added new texts from directory: %s' % texts_dir)

    
  #ładuje nowe teksty i fragmenty do bazy z podanego katalogu
  @transaction.atomic
  def __load(self, texts_dir):
    global TAGER_USER
    try:
      TAGER_USER = User.objects.get(username = 'tager')
    except:
      self.stdout.write('There is no "tager" user in database. Set it and run program again')
      return 1
    try:
      s = time.clock()
      self.stdout.write(u'Loading texts from {}...'.format(texts_dir))
#      text_list = os.listdir(unicode(texts_dir))
      for dirname, subdirs, files in os.walk(texts_dir):
        if ('text.xml' in files) and ('ann_morphosyntax.xml' in files):
          dir_split = unicode(dirname).rsplit('/',2)
          if dir_split[-1] == 'selection':
            text_id = dir_split[-2]
          else:
            text_id = dir_split[-1]
          self.stdout.write(u'Processing text: {}'.format(text_id))
          text_exists = Text.objects.filter(text_name = text_id).exists()
          if text_exists == False:
#            with io.open(os.path.join(dirname, 'header.xml'), 'r', encoding='utf-8') as header_file:
#              header_text = header_file.read()
#            text_obj = Text(text_name = text_id, header = header_text, dirname = dirname)
#            text_obj.save()
            self.stdout.write(u'There is no \'{}\' text in database. Skipping...'.format(text_id))
            continue
          else:
            text_obj = Text.objects.get(text_name = text_id) 
          self.__load_excerpts(dirname, text_obj)
      self.stdout.write("Loading time: {}".format(time.clock()-s))
      return 0
    except:
      self.stdout.write(u'Failed to read text info from %s' % texts_dir)
      raise

  #zwraca true jeśli segment podany jako argument jest ostatni
  #(lub może być ostatni - choice) w ramach zdania
  def is_last(self, xml_seg):
    ret = False
    xml_choice = xml_seg.xpath('ancestor::tei:choice', namespaces = namespaces)
    if len(xml_choice) == 0: #segment bezpośrednio w próbce
      if xml_seg.getnext() is None:
        ret = True
    elif xml_choice[0].getnext() is None: #segment gdzieś wewnątrz ostatniego choice w zdaniu
      if xml_seg.getparent() == xml_choice: #segment bezpośrednio w choice
        ret = True
      elif xml_seg.getnext() is None: #segment musi być w paren, czyli trzeba złapać ostatni
        ret = True
    return ret

  def tag_wo_ns(self, elem):
    return elem.tag.rsplit("}", 1)[-1]
    
    
  def make_interps(self, seg_filter, xml_interps):
    try:
      os.remove('tagset_err.txt')
    except:
      pass
    interps = []
    for xml_lex in xml_interps:
      lex = {}
      lex['base'] = xml_lex.xpath('./tei:f[@name="base"]/tei:string', namespaces = namespaces)[0].text
      lex['ctag'] = xml_lex.xpath('./tei:f[@name="ctag"]/tei:symbol/@value', namespaces = namespaces)[0]
      lex['msd'] = []
      
      xml_msds = xml_lex.xpath('./tei:f[@name="msd"]//tei:symbol', namespaces = namespaces)
      for xml_msd in xml_msds:
        msd = {}
        msd['val'] = xml_msd.get('value')
        for e in msd['val'].split(":"):
          if (e != "") and (e not in SORT_ORDER):
            t_err = io.open('tagset_err.txt', 'a')
            t_err.write(u'base: {}, ctag: {}, msd: {}, kod: {}\n'.format(lex['base'], lex['ctag'], msd['val'], e))
            t_err.close()
        msd['manual'] = False
        lex['msd'].append(msd)
#      lex['msd'].sort(key=lambda elem : ":".join(map(lambda e : e.replace(e, '{:02d}'.format(SORT_ORDER.index(e)+1)) if e in SORT_ORDER else "00", elem['val'].split(":"))) if len(elem['val']) > 0 else "00" )
         
      interps.append(lex)

      # Jeśli jedyną interpretacją jest interpuncja (ctag=interp), to ustaw ją od razu w bazie
      if (len(xml_interps) == 1) and (len(xml_msds) == 1) and (lex['ctag'] == "interp"):
        seg_filter['base'] = lex['base']
        seg_filter['ctag'] = 'interp'
        seg_filter['radio_id'] = '0.0'
        
#    interps.sort(key=lambda elem : elem['base'] + ':' + elem['ctag'])
    
#    seg_filter['interps'] = json.dumps(interps)
    return interps
    

  def find_morph(self, seg_id, morph_index, morph_list):
    m_index = morph_index
    while (m_index < len(morph_list)) and (morph_list[m_index].get('corresp','') <> "ann_segmentation.xml#{}".format(seg_id)):
      m_index += 1
    if m_index == len(morph_list):
      return (morph_index, []) 
    else:
      return (m_index, [morph_list[m_index]])
    
    
  def make_tagger_segment(self, exc, entry_id, xml_seg, segs, choice_id, paren_id, xml_morph_seg):

    seg_id = xml_seg.get('{http://www.w3.org/XML/1998/namespace}id')
    seg_id_nr = int(seg_id.rsplit('-', 1)[0].rsplit('.', 1)[1])
    if seg_id_nr != segs:
      self.stdout.write(u'Different segment ordering: {} in xml_id and {} in order'.format(seg_id_nr, segs))
      return False
      
    seg_filter = {}
    seg_filter['excerpt'] = exc
    seg_filter['xml_id'] = entry_id
    
    corresp = xml_seg.get('corresp')
    if corresp is not None:
      seg_filter['corresp_elem'], seg_filter['corresp_pos'], seg_filter['corresp_len'] = corresp.split('(')[1][:-1].split(',')
    #jeśli trzeba, ustaw pole choice_id
    if choice_id != -1:
      seg_filter['choice_id'] = choice_id
    else:
      seg_filter['choice_id__isnull'] = True
    #jeśli trzeba, ustaw pole paren_id
    if paren_id != -1:
      seg_filter['paren_id'] = paren_id
    else:
      seg_filter['paren_id__isnull'] = True
    #jeśli trzeba, ustaw pole 'nps'
    xml_nps = xml_seg.get('{http://www.nkjp.pl/ns/1.0}nps')
    if xml_nps is not None:
      seg_filter['nps'] = True
    else:
      seg_filter['nps'] = False
    #jeśli trzeba, ustaw pole 'type'
    xml_type = xml_seg.get('type')
    if xml_type is not None:
      seg_filter['seg_type'] = xml_type
    else:
      seg_filter['seg_type'] = ''
      
    #jeśli ostatni w zdaniu, ustaw koniec zdania, ewentualnie koniec ab
    if self.is_last(xml_seg):
      seg_filter['sent_end'] = True
    else:
      seg_filter['sent_end'] = False

    seg_filter['korbeusz'] = True
    seg_filter['is_rejected'] = Reject_status.UNSET
    seg_filter['is_hidden'] = False
    seg_filter['orth_manual'] = False
    seg_filter['manual'] = False
    seg_filter['base'] = ''
    seg_filter['ctag'] = ''
    seg_filter['msd'] = ''
    seg_filter['radio_id'] = ''
    seg_filter['is_conflict'] = False
    seg_filter['ref_id'] = 0
    seg_filter['ref_rejected'] = Reject_status.UNSET
    seg_filter['user__isnull'] = True 
    
    #dla elementów niepustych (mających opis w ann_morphosyntax) ustaw pola 'orth' i 'translit'
    if len(xml_morph_seg) <> 0:
      seg_filter['translit'] = xml_morph_seg[0].xpath('./tei:fs/tei:f[@name="translit"]/tei:string', namespaces = namespaces)[0].text
      if xml_type in NO_INTERP:
        seg_filter['orth'] = seg_filter['translit']
      else:
        seg_filter['orth'] = xml_morph_seg[0].xpath('./tei:fs/tei:f[@name="orth"]/tei:string', namespaces = namespaces)[0].text
      # Nie należy sprawdzać interpretacji - w wersji tagera nie jest sortowana!!!
      # przypisz interpretacje - dla elementów nieinterpretowanych pusta interpretacja
#      if xml_type in NO_INTERP:
#        seg_filter['interps'] = ''
#      else:
#        seg_filter = self.make_interps_JSON(seg_filter, xml_morph_seg[0].xpath('./tei:fs/tei:f[@name="interps"]/tei:fs[@type="lex"]', namespaces = namespaces))
      if xml_type in NO_INTERP:
        seg_filter['interps'] = ''
      else:
        seg_interps = self.make_interps(seg_filter, xml_morph_seg[0].xpath('./tei:fs/tei:f[@name="interps"]/tei:fs[@type="lex"]', namespaces = namespaces))
      
    seg_list = Segment.objects.filter(**seg_filter)

    if len(seg_list) == 1:
      # Sprawdź, czy nie jest to element choice'a, a więc czy nie trzeba ustawić Reject_status
      if len(xml_seg.xpath('ancestor::tei:choice', namespaces = namespaces)) > 0:
        # Sprawdź, czy segment nie został odrzucony - wykluczony wariant segmentacyjny
        seg_rejected = xml_seg.get('{http://www.nkjp.pl/ns/1.0}rejected')
        if seg_rejected == 'true':
          reject = Reject_status.YES
        else:
          reject = Reject_status.NO
      else:
        reject = Reject_status.UNSET
      
      if (reject == Reject_status.YES) or (xml_type in NO_INTERP):
        # Utwórz kopię znalezionego rekordu
        seg = seg_list[0]
        seg.pk = None
        seg.user = TAGER_USER
        seg.is_rejected = reject
        seg.seg_change = timezone.now()
        seg.save()          
        return True
      else: 
        disamb_list = xml_morph_seg[0].xpath('./tei:fs/tei:f[@name="disamb"]/tei:fs[@type="tool_report"]/tei:f[@name="choice"]', namespaces = namespaces)
        base_ctag_list = xml_morph_seg[0].xpath('./tei:fs/tei:f[@name="disamb"]/tei:fs[@type="tool_report"]/tei:f[@name="interpretation"]/tei:string', namespaces = namespaces)
        if (len(disamb_list) == 1) and (len(base_ctag_list) == 1):
          disamb = disamb_list[0].get('fVal')
          # fVal może być pustym napisem, jeśli to interpretacja dodana przez tager - trzeba wtedy ustawić ręczną interpretację
          if disamb == '':
            base_ctag = base_ctag_list[0].text # uwaga na base = ":"!
            if base_ctag[0:2] == '::':
              base_ctag = base_ctag[2:].split(':', 1)
              base_ctag.insert(0, ':')
            else:
              base_ctag = base_ctag.split(':', 2)  
            seg_interps.insert(0, {'base': base_ctag[0], 'ctag': base_ctag[1], 'msd':[{'manual': True, 'val': base_ctag[2] if len(base_ctag) > 2 else ''}]})
            # Utwórz kopię znalezionego rekordu
            seg = seg_list[0]
            seg.pk = None
            seg.user = TAGER_USER
            seg.is_rejected = reject
            seg.base = base_ctag[0]
            seg.ctag = base_ctag[1]
            seg.msd = base_ctag[2] if len(base_ctag) > 2 else ''
            seg.interps = json.dumps(seg_interps)
            seg.radio_id = '0.0'
            seg.manual = True
            seg.seg_change = timezone.now()
            seg.save()          
            return True
          else:
            disamb = disamb.rsplit('-', 1)[0].rsplit('.', 2)
            disamb_lex = int(disamb[1]) - 1
            disamb_msd = int(disamb[2]) - 1
            base_ctag = base_ctag_list[0].text # uwaga na base = ":"!
            if base_ctag[0:2] == '::':
              base_ctag = base_ctag[2:].split(':', 1)
              base_ctag.insert(0, ':')
            else:
              base_ctag = base_ctag.split(':', 2)  
            # Utwórz kopię znalezionego rekordu
            seg = seg_list[0]
            seg.pk = None
            seg.user = TAGER_USER
            seg.is_rejected = reject
            seg.interps = json.dumps(seg_interps)
            msd = base_ctag[2] if len(base_ctag) > 2 else ''
            if (seg_interps[disamb_lex]['base'] != base_ctag[0]) or (seg_interps[disamb_lex]['ctag'] != base_ctag[1]) or (seg_interps[disamb_lex]['msd'][disamb_msd]['val'] != msd):
              self.stdout.write(u'Incorrect disamb in segment: {}'.format(seg_id))
              return False
            seg.base = base_ctag[0]
            seg.ctag = base_ctag[1]
            seg.msd = msd
            seg.radio_id = '{}.{}'.format(disamb_lex, disamb_msd)
            seg.seg_change = timezone.now()
            seg.save()          
            return True
        else:
#          print "disamb_list: {}".format(xml_morph_seg[0].get('{http://www.w3.org/XML/1998/namespace}id'))
          self.stdout.write(u'Problem with number of disambs: {} not 1'.format(len(disamb_list)))
          return False
    else:
      self.stdout.write(u'Found {} segments in database, seg_filter: {}'.format(len(seg_list), seg_filter))
      return False
    
  #ładuje nowe fragmenty do bazy z podanego katalogu
  @transaction.atomic
  def __load_excerpts(self, dirname, text_obj):
    corresp = ''
    try:
      #self.stdout.write(u'Loading excerpts: %s.' % text_obj.text_name)
      morph_tree = etree.parse(os.path.join(dirname, 'ann_morphosyntax.xml'))
      morph_seg_list = morph_tree.xpath('/tei:teiCorpus/tei:TEI/tei:text/tei:body/tei:p/tei:s/tei:seg', namespaces = namespaces)
      morph_index = 0
      seg_tree = etree.parse(os.path.join(dirname, 'ann_segmentation.xml'))
      
      xml_excerpts = seg_tree.xpath('/tei:teiCorpus/tei:TEI/tei:text/tei:body/tei:p', namespaces = namespaces)
      for xml_p in xml_excerpts:
        res = True
        p_id = xml_p.get('{http://www.w3.org/XML/1998/namespace}id')
        if p_id is None:
          raise ValueError(u'Excerpt without id in {}!'.format(os.path.join(dirname, 'ann_segmentation.xml')))
        else:
          exc_id = int(p_id.rsplit('-', 1)[0].rsplit('_', 1)[1])
          if exc_id > 0:
            self.stdout.write(u'Excerpt {}'.format(exc_id))

            #s = time.clock()

            # Znajdź odpowiedni obiekt Excerpt w bazie. Może go nie być - wtedy taką próbkę pomijamy
            try:
              exc = Excerpt.objects.get(text = text_obj, xml_id = exc_id)
            except:
              self.stdout.write(u'There is no excerpt {} from text {} in database. Skipping...'.format(exc_id, text_obj.text_name))
              continue
              
            # Sprawdź, czy taka próbka nie jest już przydzielona dwóm współanotatorom
            # lub czy nie jest już przydzielona "tager"owi
            work_dets = Workdetail.objects.filter(excerpt = exc)
            if len(work_dets.filter(user = TAGER_USER)) > 0:
              self.stdout.write(u'Excerpt {} from text {} already imported. Skipping...'.format(exc_id, text_obj.text_name))
              continue
              
            if len(work_dets) == 2:
              self.stdout.write(u'Excerpt {} from text {} already assigned to two annotators. Skipping...'.format(exc_id, text_obj.text_name))
              continue
            elif len(work_dets) == 1:
              if work_dets[0].work_status > Work_status.REWAITING:
                self.stdout.write(u'Excerpt {} from text {} already done. Skipping...'.format(exc_id, text_obj.text_name))
                continue
              
            # Sprawdź, czy w bazie jest taka sama liczba segmentów w oryginalnej próbce
            if len(Segment.objects.filter(excerpt = exc, user__isnull = True)) != len(xml_p.xpath('.//tei:seg', namespaces = namespaces)):
              self.stdout.write(u'Excerpt {} from text {} has different length (in segments) in database {} and text {}. Skipping...'.format(exc_id, text_obj.text_name, len(Segment.objects.filter(excerpt = exc, user__isnull = True)), len(xml_p.xpath('.//tei:seg', namespaces = namespaces))))
              continue
              
            
            segs = 1  #numer kolejny segmentu w ramach próbki

            for entry_id, xml_segentry in enumerate(xml_p.xpath('./tei:s/*', namespaces = namespaces)): #w zdaniu 's' może być segment lub choice
              #xml_corresp = xml_seg.get('corresp').split('.')[1] #na indeksie [0] jest 'text' z napisu 'text.xml'
              
              tag = self.tag_wo_ns(xml_segentry)
              #self.stdout.write(u'Entry_id: {}, tag: {}'.format(entry_id + 1, tag))

              if tag == 'seg':
                #self.stdout.write(u'Segment: {}, morph_index: {}'.format(segs, morph_index))
                i, morph_seg = self.find_morph(xml_segentry.get('{http://www.w3.org/XML/1998/namespace}id'), morph_index, morph_seg_list)
                res = res and self.make_tagger_segment(exc, entry_id + 1, xml_segentry, segs, -1, -1, morph_seg) # segment bezpośrednio w 's'
                morph_index = i
                segs += 1
              elif tag == 'choice':
                for choice_id, xml_choice in enumerate(xml_segentry):
                  tag_choice = self.tag_wo_ns(xml_choice)
                  if tag_choice == 'seg':
                    #self.stdout.write(u'Segment: {}, morph_index: {}'.format(segs, morph_index))
                    i, morph_seg = self.find_morph(xml_choice.get('{http://www.w3.org/XML/1998/namespace}id'), morph_index, morph_seg_list)
                    res = res and self.make_tagger_segment(exc, entry_id + 1, xml_choice, segs, choice_id + 1, -1, morph_seg) # segment bezpośrednio w 'choice' - wszystkie takie dostają ten sam entry_id
                    morph_index = i
                    segs += 1
                  elif tag_choice == 'paren':
                    for paren_id, xml_paren in enumerate(xml_choice):
                      tag_paren = self.tag_wo_ns(xml_paren)
                      if tag_paren == 'seg':
                        #self.stdout.write(u'Segment: {}, morph_index: {}'.format(segs, morph_index))
                        i, morph_seg = self.find_morph(xml_paren.get('{http://www.w3.org/XML/1998/namespace}id'), morph_index, morph_seg_list)
                        res = res and self.make_tagger_segment(exc, entry_id + 1, xml_paren, segs, choice_id + 1, paren_id + 1, morph_seg) # segment w 'paren'
                        morph_index = i
                        segs += 1
                      else:
                        raise ValueError(u'{} w paren, tekst: {}, nr segmentu: {}'.format(tag_paren, text_obj.text_name, segs))
                  else:
                    raise ValueError(u'{} w choice, tekst: {}, nr segmentu: {}'.format(tag_choice, text_obj.text_name, segs))
              else:
                raise ValueError(u'{} w zdaniu, tekst: {}, nr segmentu: {}'.format(tag, text_obj.text_name, segs))
              
              if not res:
                raise ValueError(u'Wystąpiły problemy z dopasowaniem segmentów w próbce {} tekstu {}'.format(exc_id, text_obj.text_name))

            if Segment.objects.filter(excerpt = exc, user__isnull = True).count() == Segment.objects.filter(excerpt = exc, user = TAGER_USER).count(): # test na wszelki wypadek
              if len(work_dets) == 1:
                wd_other = work_dets[0]
                wd_stat = wd_other.work_status
                if wd_stat == Work_status.ASSIGNED:
                  tager_status = Work_status.WAITING
                elif wd_stat == Work_status.WAITING:
                  tager_status = Work_status.ASSIGNED
                elif wd_stat == Work_status.REASSIGNED:
                  tager_status = Work_status.REWAITING
                elif wd_stat == Work_status.REWAITING:
                  tager_status = Work_status.REASSIGNED                  
              else:
                wd_stat = None
                # W normalnym trybie zmienić na WAITING, jeśli tylko jeden obieg z superanotacją - REWAITING
                tager_status = Work_status.WAITING
              
              wd = Workdetail(excerpt = exc, user = TAGER_USER, work_status = tager_status, status_change = timezone.now())
              wd.save()
              if (wd_stat == Work_status.WAITING) or (wd_stat == Work_status.REWAITING):
                checkExcerpts(exc, wd, wd_other)
            else:
              raise ValueError(u'Różne liczby segmentów po zaimportowaniu wersji tagera w próbce {} z tekstu {}'.format(exc_id, text_obj.text_name))

            #self.stdout.write("czas ladowania: {}".format(time.clock()-s))  
    
          
    except etree.LxmlError:
      fail_str = u'Failed to read xml tree for %s' % text_obj.text_name
      self.stdout.write('{}'.format(fail_str))
      raise
    except ValueError as err:
      fail_str = err.args
      self.stdout.write('{}'.format(fail_str))
      raise
    except:
      fail_str = u'Failed to process %s, unexpected error.' % text_obj.text_name
      self.stdout.write('{}'.format(fail_str))
      raise
