# -*- coding: utf-8 -*-
from django.core.management.base import BaseCommand, CommandError
from django.db import transaction
from annotation.models import Text, Excerpt, Segment
import os, io
from lxml import etree
import json
import time


namespaces = {
  'tei': 'http://www.tei-c.org/ns/1.0',
  'xi': 'http://www.w3.org/2001/XInclude',
  'nkjp': 'http://www.nkjp.pl/ns/1.0'
}


'''
Aktualizowanie xmlowych headerów przy tekstach.
Jako parametr należy podać ścieżkę do oryginalnych
plików z Wczytywacza. Skrypt przetworzy je na
wersję dla ręcznych próbek.
'''

class Command(BaseCommand):
  help = 'Update xml headers in Text table.'
  missing_args_message = 'Please provide "texts_dir" argument'
  
  def add_arguments(self, parser):
    # Positional argument
    parser.add_argument('texts_dir')


  def handle(self, *args, **options):
    #print options
    texts_dir = unicode(options.get('texts_dir'))
      
    self.stdout.write(u'Updating headers from {}...'.format(texts_dir))
    self.updateHeaders(texts_dir)
    self.stdout.write('Headers successfully updated.')


  @transaction.atomic
  def updateHeaders(self, texts_dir):
    try:
      s = time.clock()
      text_list = os.listdir(unicode(texts_dir))
      for txt in Text.objects.all():
        header_path = os.path.join(unicode(texts_dir), txt.text_name, 'header.xml')
        if os.path.exists(header_path):
          self.stdout.write(u'Processing text: {}'.format(txt.text_name))
          header_tree = etree.parse(header_path)	
          tei_head = header_tree.getroot()
          tei_id = tei_head.get('{http://www.w3.org/XML/1998/namespace}id')
          tei_head.set('{http://www.w3.org/XML/1998/namespace}id', 'RECZNA_' + tei_id) #zmiana id w headerze próbki
          tei_publStmt = tei_head.find('.//tei:publicationStmt', namespaces = namespaces)
          if tei_publStmt is not None:
            tei_publStmt.set("{%s}" % namespaces['nkjp'] + 'subcorpus', "RECZNA_KORBA")

          txt.header = etree.tostring(header_tree, pretty_print=True, encoding='UTF-8', xml_declaration=True)
          txt.save()
        else:
          self.stdout.write(u'There is no header in {}/{}'.format(texts_dir, txt.text_name))
      self.stdout.write("Loading time: {}".format(time.clock()-s))
    except:
      self.stdout.write(u'Failed updating headers from {}.'.format(texts_dir))
      raise

