# -*- coding: utf-8 -*-
from django.db import models
from django.utils import timezone
from django.contrib.auth.models import User, Group, Permission

class Role(object):
  ANNOTATOR = 0
  REFEREE = 1
  CONTROLLER = 2
  ACCOUNTANT = 3

ROLE_CHOICES = {
  Role.ANNOTATOR: u'Anotator',
  Role.REFEREE: u'Superanotator',
  Role.CONTROLLER: u'Kontroler',
  Role.ACCOUNTANT: u'Księgowy'
}

class User_profile(models.Model):
  user = models.OneToOneField(User)
  role = models.IntegerField(choices=ROLE_CHOICES.items(), default=Role.ANNOTATOR)
  
  def isReferee(self):
    if self.role == Role.REFEREE:
      return 1
    else:
      return 0

  def isReferee_or_Control(self):
    if (self.role == Role.REFEREE) or (self.role == Role.CONTROLLER):
      return 1
    else:
      return 0
  
  def isReferee_or_Accountant(self):
    if (self.role == Role.REFEREE) or (self.role == Role.ACCOUNTANT):
      return 1
    else:
      return 0

  def isReferee_or_Annot(self):
    if (self.role == Role.REFEREE) or (self.role == Role.ANNOTATOR):
      return 1
    else:
      return 0
  
  def __unicode__(self):
		return self.user.username


class Reject_status(object):
  YES = -1
  UNSET = 0
  NO = 1

REJECT_STATUS_CHOICES = {
  Reject_status.YES: u'Segment usunięty (jako składnik "choice").',
  Reject_status.UNSET: u'Segment nie ma ustalonego statusu.',
  Reject_status.NO: u'Segment zatwierdzony (jako składnik "choice").'
  }
  
class Work_status(object):
  UNASSIGNED = 0
  ASSIGNED = 1
  WAITING = 2
  REASSIGNED = 3
  REWAITING = 4
  TO_REFEREE = 5
  DONE = 6
  REF_DONE = 7

WORK_STATUS_CHOICES = {
  Work_status.UNASSIGNED: u'Próbka nieprzypisana',
  Work_status.ASSIGNED: u'Próbka przypisana do anotatora',
  Work_status.WAITING: u'Próbka czeka na drugiego anotatora',
  Work_status.REASSIGNED: u'Próbka zwrócona do poprawki',
  Work_status.REWAITING: u'Próbka czeka na drugiego anotatora po poprawkach',
  Work_status.TO_REFEREE: u'Próbka przekazana do rozstrzygnięcia do superanotatora',
  Work_status.DONE: u'Próbka zakończona',
  Work_status.REF_DONE: u'Próbka zakończona po superanotacji'
}
  
#tekst (wiele probek)
class Text(models.Model):
  text_name = models.CharField(db_index = True, max_length=200) #identyfikator z korby
  header = models.TextField(blank=True) #zawartość pliku header.xml dla danego tekstu
  dirname = models.CharField(max_length=128) 
  #import_date = models.DateTimeField('data importu', auto_now = True)
  import_date = models.DateTimeField('data importu', default=timezone.now)

  def __unicode__(self):
    return self.text_name

#probka (wiele segmentow)
class Excerpt(models.Model):
  text = models.ForeignKey(Text)
  xml_id = models.IntegerField() # sam numer probki, czyli zamiast "segm_1-p" tylko "1"
  xml_content = models.TextField(blank=True) #treść próbki z pliku text.xml (cały <div>)
  import_date = models.DateTimeField('data importu', default=timezone.now)

  def __unicode__(self):
    return self.text.text_name + '.seg_' + str(self.xml_id)
    
  def getStatus(self, user):
    return WORK_STATUS_CHOICES[int(self.details.get(user = user).work_status)]
    
  def getUsers(self):
    exc_users = [det.user for det in self.details.filter(excerpt = self).order_by('user')]
    return exc_users

  def getReferee(self):
    exc_users = [det.ref_user for det in self.details.filter(excerpt = self).order_by('ref_user')]
    if exc_users:
      return exc_users[0]
    else:
      return None

  def getConflicts(self, foruser):
    return Segment.objects.filter(excerpt = self, user = foruser, is_conflict = True).exclude(is_rejected = Reject_status.YES).count()
  
  def getNotes(self, foruser):
    return Workdetail.objects.get(excerpt = self, user = foruser).notes

  
#szczegóły pracy anotatora (odniesienie do próbki)
class Workdetail(models.Model):
  excerpt = models.ForeignKey(Excerpt, related_name='details')
  work_status = models.IntegerField(choices=WORK_STATUS_CHOICES.items(), default=Work_status.UNASSIGNED) # status próbki u anotatora
  status_change = models.DateTimeField('data zmiany statusu', default=timezone.now)
  user = models.ForeignKey(User) # anotator przypisany do próbki
  notes = models.TextField(blank=True) #uwagi anotatora do próbki
  ref_user = models.ForeignKey(User, blank=True, null = True, default=None, related_name='referee') # superanotator przypisany do próbki
  pay_date = models.DateField(null=True, blank=True, default=None) # data rozliczenia próbki dla anotatora
  pay_account = models.ForeignKey(User, blank=True, null = True, default=None, related_name='pay_account') # użytkownik zatwierdzający anotację 
  ref_pay_date = models.DateField(null=True, blank=True, default=None) # data rozliczena próbki dla superanotatora
  ref_pay_account = models.ForeignKey(User, blank=True, null = True, default=None, related_name='ref_pay_account') # użytkownik zatwierdzający superanotację 

  def __unicode__(self):
    return self.excerpt.text.text_name + '.exc_' + str(self.excerpt.xml_id) + '.user_' + str(self.user.username) + '.status_' + str(self.work_status)

#class Segment(SegEntry):
class Segment(models.Model):
  '''
  xml_id w tym miejscu to sam numer kolejny segmentu. Uwaga! Numer ciagly 
  w ramach probki, a nie zdania. Zdania nie stanowia tu oddzielnej tabeli
  a koniec zdania jest po prostu cecha segmentu (sent_end). Chodzi o latwa
  zmiane podzialu na zdania.
  '''
  excerpt = models.ForeignKey(Excerpt, related_name='segs')
  xml_id = models.IntegerField(db_index=True) # może się powtarzać dla segmentów w choice
  corresp_elem = models.CharField(max_length=200, blank=True) # corresp - id elementu nadrzędnego
  corresp_pos = models.IntegerField(blank=True, null=True) # corresp - pozycja początku segmentu
  corresp_len = models.IntegerField(blank=True, null=True) # długość segmentu w znakach
  choice_id = models.IntegerField(blank=True, null=True) # w ramach jednej próbki - identyfikator kolejnego 'choice', do którego należy segment
  paren_id = models.IntegerField(blank=True, null=True) # w ramach jednego choice'a - identyfikator kolejnego 'paren', do którego należy segment
  is_rejected = models.IntegerField(choices=REJECT_STATUS_CHOICES.items(), default=Reject_status.UNSET) 
  is_hidden = models.BooleanField(default=False) # czy ukryty - możliwe do ustawienia dla początkowego i końcowego fragmentu próbki
  korbeusz = models.BooleanField(default=False) # czy segment pochodzi z Korbeusza
  sent_end = models.BooleanField(default=False) 
  nps = models.BooleanField(default=False)
  seg_type = models.CharField(max_length=20, blank=True)
  orth = models.CharField(db_index=True, max_length=128, blank=True)
  orth_manual = models.BooleanField(default=False) # czy poprawiona ręcznie transkrypcja
  translit = models.CharField(max_length=128, blank=True)
  interps = models.TextField() # zapis JSON wariantów interpretacyjnych
  base = models.CharField(max_length=250, blank=True) # wybrana interpretacja - lemat
  ctag = models.CharField(max_length=20, blank=True) # wybrana interpretacja - rodzaj
  msd = models.CharField(max_length=250, blank=True) # wybrana interpretacja
  manual = models.BooleanField(default=False)
  radio_id = models.CharField(max_length=10, blank=True) #indeks wybranej interpretacji w formacie <interp>.<msd>
  user = models.ForeignKey(User, blank=True, null = True, default=None) # anotator przypisany do segmentu
  seg_change = models.DateTimeField('data zmiany segmentu', blank=True, null=True, default=None)
  is_conflict = models.BooleanField(default=False) # czy na danym segmencie jest konflikt
  ref_id = models.IntegerField(db_index=True, default=0) # odpowiednik xml_id - grupuje wspólny zbiór segmentów dla wyboru w superanotacji, 0 dla wersji drugiego anotatora, jeśli były zgodne
  ref_rejected = models.IntegerField(choices=REJECT_STATUS_CHOICES.items(), default=Reject_status.UNSET)  # wskazuje wybraną przez superanotatora wersję 
   

  
  def __unicode__(self):
    if self.orth:
      s = self.orth
    elif self.seg_type:
      s = '<{}>'.format(self.seg_type)
    else:
      s = '<xxx>'
    return self.excerpt.text.text_name + '.exc_' + str(self.excerpt.xml_id) + '_' + s

