
function validMsd(ctag_text, msd_text){

  var liczba_lst = "(sg|du|pl)([\.](sg|du|pl))*";
  var przypadek_lst = "(nom|gen|dat|acc|inst|loc|voc)([\.](nom|gen|dat|acc|inst|loc|voc))*";
  var rodzaj_lst = "(0|((manim1|manim2|mnanim|m|n|f|p1|p2)([\.](manim1|manim2|mnanim|m|n|f|p1|p2))*))";
  var osoba_lst = "(pri|sec|ter)([\.](pri|sec|ter))*";
  var osoba12_lst = "(pri|sec|pri[\.]sec|sec[\.]pri)";
  var stopien_lst = "(pos|com|sup)([\.](pos|com|sup))*";
  var aspekt_lst = "(imperf|perf|biasp)([\.](imperf|perf|biasp))*";
  var negacja_lst = "(aff|neg|aff[\.]neg|neg[\.]aff)";
  var akcent_lst = "(akc|nakc|zneut)([\.](akc|nakc|zneut))*";
  var poprzyimk_lst = "(praep|npraep|praep[\.]npraep|npraep[\.]praep)";
  var aglutyn_lst = "(agl|nagl|agl[\.]nagl|nagl[\.]agl)";
  var wokal_lst = "(wok|nwok|wok[\.]nwok|nwok[\.]wok)";
  var kropkow_lst = "(pun|npun|pun[\.]npun|npun[\.]pun)";

  var tagset_list = {
  "adj": {r: liczba_lst + ":" + przypadek_lst + ":" + rodzaj_lst + ":" + stopien_lst, t: "<em>[liczba]</em><strong>:</strong><em>[przypadek]</em><strong>:0|</strong><em>[rodzaj]</em><strong>:</strong><em>[stopien]</em>"},
  "adja": {r: "", t: "<em>[]</em>"},
  "adjb": {r: liczba_lst + ":" + przypadek_lst + ":" + rodzaj_lst + ":pos", t: "<em>[liczba]</em><strong>:</strong><em>[przypadek]</em><strong>:0|</strong><em>[rodzaj]</em><strong>:pos"},
  "adjnum": {r: liczba_lst + ":" + przypadek_lst + ":" + rodzaj_lst + ":pos", t: "<em>[liczba]</em><strong>:</strong><em>[przypadek]</em><strong>:0|</strong><em>[rodzaj]</em><strong>:pos"},
  "adv": {r: "(" + stopien_lst + ")?", t: "(<em>[stopien]</em>)?"},
  "advnum": {r: "", t: "<em>[]</em>"},
  "aglt": {r: liczba_lst + ":" + osoba_lst + ":" + aspekt_lst + ":" + wokal_lst, t: "<em>[liczba]</em><strong>:</strong><em>[osoba]</em><strong>:</strong><em>[aspekt]</em><strong>:</strong><em>[wokal]</em>"},
  "agltaor": {r: liczba_lst + ":pri:imperf:" + wokal_lst, t: "<em>[liczba]</em><strong>:pri:imperf:</strong><em>[wokal]</em>"},
  "bedzie": {r: liczba_lst + ":" + osoba_lst + ":" + aspekt_lst, t: "<em>[liczba]</em><strong>:</strong><em>[osoba]</em><strong>:</strong><em>[aspekt]</em>"},
  "brev": {r: kropkow_lst, t: "<em>[kropkow]</em>"},
  "comp": {r: "", t: "<em>[]</em>"},
  "conj": {r: "", t: "<em>[]</em>"},
  "dig": {r: "", t: "<em>[]</em>"},
  "fin": {r: liczba_lst + ":" + osoba_lst + ":" + aspekt_lst, t: "<em>[liczba]</em><strong>:</strong><em>[osoba]</em><strong>:</strong><em>[aspekt]</em>"},
  "frag": {r: "", t: "<em>[]</em>"},
  "fut": {r: liczba_lst + ":" + osoba_lst + ":" + aspekt_lst, t: "<em>[liczba]</em><strong>:</strong><em>[osoba]</em><strong>:</strong><em>[aspekt]</em>"},
  "ger": {r: liczba_lst + ":" + przypadek_lst + ":n:" + aspekt_lst + ":" + negacja_lst, t: "<em>[liczba]</em><strong>:</strong><em>[przypadek]</em><strong>:n:</strong><em>[aspekt]</em><strong>:</strong><em>[negacja]</em>"},
  "ign": {r: "", t: "<em>[]</em>"},
  "ignndm": {r: "", t: "<em>[]</em>"},
  "imps": {r: aspekt_lst, t: "<em>[aspekt]</em>"},
  "impt": {r: liczba_lst + ":" + osoba_lst + ":" + aspekt_lst, t: "<em>[liczba]</em><strong>:</strong><em>[osoba]</em><strong>:</strong><em>[aspekt]</em>"},
  "inf": {r: aspekt_lst, t: "<em>[aspekt]</em>"},
  "interj": {r: "", t: "<em>[]</em>"},
  "interp": {r: "", t: "<em>[]</em>"},
  "num": {r: liczba_lst + ":" + przypadek_lst + ":" + rodzaj_lst, t: "<em>[liczba]</em><strong>:</strong><em>[przypadek]</em><strong>:0|</strong><em>[rodzaj]</em>"},
  "numcol": {r: liczba_lst + ":" + przypadek_lst + ":" + rodzaj_lst, t: "<em>[liczba]</em><strong>:</strong><em>[przypadek]</em><strong>:0|</strong><em>[rodzaj]</em>"},
  "pact": {r: liczba_lst + ":" + przypadek_lst + ":" + rodzaj_lst + ":" + aspekt_lst + ":" + negacja_lst + ":" + stopien_lst, t: "<em>[liczba]</em><strong>:</strong><em>[przypadek]</em><strong>:0|</strong><em>[rodzaj]</em><strong>:</strong><em>[aspekt]</em><strong>:</strong><em>[negacja]</em><strong>:</strong><em>[stopien]</em>"},
  "pacta": {r: "", t: "<em>[]</em>"},
  "pactb": {r: "sg:nom:f:" + aspekt_lst + ":" + negacja_lst + ":" + stopien_lst, t: "sg:nom:f:</strong><em>[aspekt]</em><strong>:</strong><em>[negacja]</em><strong>:</strong><em>[stopien]</em>"},
  "pant": {r: aspekt_lst, t: "<em>[aspekt]</em>"},
  "pcon": {r: aspekt_lst, t: "<em>[aspekt]</em>"},
  "plusq": {r: liczba_lst + ":" + rodzaj_lst + ":imperf", t: "<em>[liczba]</em><strong>:0|</strong><em>[rodzaj]</em><strong>:imperf"},
  "ppas": {r: liczba_lst + ":" + przypadek_lst + ":" + rodzaj_lst + ":" + aspekt_lst + ":" + negacja_lst + ":" + stopien_lst, t: "<em>[liczba]</em><strong>:</strong><em>[przypadek]</em><strong>:0|</strong><em>[rodzaj]</em><strong>:</strong><em>[aspekt]</em><strong>:</strong><em>[negacja]</em><strong>:</strong><em>[stopien]</em>"},
  "ppasb": {r: liczba_lst + ":" + przypadek_lst + ":" + rodzaj_lst + ":" + aspekt_lst + ":" + negacja_lst + ":pos", t: "<em>[liczba]</em><strong>:</strong><em>[przypadek]</em><strong>:0|</strong><em>[rodzaj]</em><strong>:</strong><em>[aspekt]</em><strong>:</strong><em>[negacja]</em><strong>:pos"},
  "ppraet": {r: liczba_lst + ":" + przypadek_lst + ":" + rodzaj_lst + ":" + aspekt_lst + ":" + negacja_lst, t: "<em>[liczba]</em><strong>:</strong><em>[przypadek]</em><strong>:0|</strong><em>[rodzaj]</em><strong>:</strong><em>[aspekt]</em><strong>:</strong><em>[negacja]</em>"},
  "ppron12": {r: liczba_lst + ":" + przypadek_lst + ":" + rodzaj_lst + ":" + osoba12_lst + "(:" + akcent_lst + ")?", t: "<em>[liczba]</em><strong>:</strong><em>[przypadek]</em><strong>:0|</strong><em>[rodzaj]</em><strong>:</strong><em>[osoba12]</em>(:<em>[akcent]</em>)?"},
  "ppron3": {r: liczba_lst + ":" + przypadek_lst + ":" + rodzaj_lst + ":ter:" + akcent_lst + ":" + poprzyimk_lst, t: "<em>[liczba]</em><strong>:</strong><em>[przypadek]</em><strong>:0|</strong><em>[rodzaj]</em><strong>:ter:</strong><em>[akcent]</em><strong>:</strong><em>[poprzyimk]</em>"},
  "praet": {r: liczba_lst + ":" + rodzaj_lst + ":" + aspekt_lst + "(:" + aglutyn_lst + ")?", t: "<em>[liczba]</em><strong>:0|</strong><em>[rodzaj]</em><strong>:</strong><em>[aspekt]</em>(:<em>[aglutyn]</em>)?"},
  "pred": {r: "", t: "<em>[]</em>"},
  "prep": {r: przypadek_lst + "(:" + wokal_lst + ")?", t: "<em>[przypadek]</em>(:<em>[wokal]</em>)?"},
  "qub": {r: "", t: "<em>[]</em>"},
  "romandig": {r: "", t: "<em>[]</em>"},
  "siebie": {r: przypadek_lst, t: "<em>[przypadek]</em>"},
  "subst": {r: liczba_lst + ":" + przypadek_lst + ":" + rodzaj_lst, t: "<em>[liczba]</em><strong>:</strong><em>[przypadek]</em><strong>:0|</strong><em>[rodzaj]</em>"},
  "winien": {r: liczba_lst + ":" + rodzaj_lst + ":" + aspekt_lst, t: "<em>[liczba]</em><strong>:0|</strong><em>[rodzaj]</em><strong>:</strong><em>[aspekt]</em>"},
  "xxx": {r: "", t: "<em>[]</em>"},
  };
    
  if (tagset_list.hasOwnProperty(ctag_text)) {
    var regex = new RegExp('^' + tagset_list[ctag_text].r + '$', "i");
    if (regex.test(msd_text)) {
      //alert(regex.exec(msd_text));
      return true;
    } else {
      $("#mod_text").html('<p>Wpisana interpretacja nie pasuje do wzoraca dla: <strong>' + ctag_text + ':</p><table class="table"><tr><td style="width:60px;">Tag:</td><td>' + msd_text + '</td></tr><tr><td style="width:60px;">Wzorzec:</td><td>' + tagset_list[ctag_text].t + '</td></tr></table>');
      $("#mod_alert").modal('show');
      return false;
    };
  } else {
    $("#mod_text").html('<p>W ręcznej anotacji nie można użyć części mowy: <strong>' + ctag_text + '</strong></p>')
    $("#mod_alert").modal('show');
    return false;
  };
}

  
