# -*- coding: utf-8 -*-
from django.shortcuts import render, get_object_or_404, redirect
from django.views.decorators.csrf import csrf_exempt, csrf_protect
from django.http import HttpResponseRedirect, HttpResponse
from django.db import transaction, connection, DatabaseError
from django.contrib.auth.decorators import login_required, user_passes_test
from .models import Text, Segment, Excerpt, Reject_status, Workdetail, Work_status, WORK_STATUS_CHOICES, User
from copy import deepcopy
from django.utils import timezone
from datetime import date
import json
import time
from django.core.serializers import serialize
from django.db.models.query import QuerySet
from django.db.models import Q, Min, Count, Max, Case, When, IntegerField
from django.core.serializers.json import DjangoJSONEncoder
from django.utils.safestring import mark_safe
import random
from .anot2_config import REF_MODE, EXC_SET, EXC_MAX, MAX_IGN, MAX_IGNNDM
import csv
from django.core.paginator import Paginator, EmptyPage, PageNotAnInteger

def jsonify(object):
  if isinstance(object, QuerySet):
    return mark_safe(serialize('json', object))
  return mark_safe(json.dumps(object))

def jsonify_a(object):
    if isinstance(object, QuerySet):
        return mark_safe(json.dumps(list(object.values()), cls=DjangoJSONEncoder))
    return mark_safe(json.dumps(object))

def getPrevNextExc(exc, usr):
  user_works = Workdetail.objects.filter(user = usr, excerpt__segs__user = usr)
  cur_work = user_works.filter(excerpt = exc)
  cur_work_stat = cur_work[0].work_status
  
  if cur_work_stat in [Work_status.ASSIGNED, Work_status.REASSIGNED]:
    work_det = user_works.filter(Q(work_status = Work_status.ASSIGNED) | Q(work_status = Work_status.REASSIGNED)).annotate(mod_date=Max('excerpt__segs__seg_change')).order_by('-work_status', 'mod_date')
  elif cur_work_stat in [Work_status.WAITING]:
    work_det = user_works.filter(work_status = Work_status.WAITING).annotate(mod_date=Max('excerpt__segs__seg_change')).order_by('mod_date')
  elif cur_work_stat in [Work_status.REWAITING, Work_status.TO_REFEREE]:
    work_det = user_works.filter(Q(work_status = Work_status.TO_REFEREE) | Q(work_status = Work_status.REWAITING)).annotate(mod_date=Max('excerpt__segs__seg_change')).order_by('-work_status', 'mod_date')
  else:
    work_det = user_works.filter(Q(work_status = Work_status.DONE) | Q(work_status = Work_status.REF_DONE)).annotate(mod_date=Max('excerpt__segs__seg_change')).order_by('-mod_date')
    
#  cur_work = list(work_det.filter(excerpt=exc))[0]
  cur_work = list(cur_work)[0]
  works_list = list(work_det)
  cur_work_ind = works_list.index(cur_work)
  
  if (cur_work_ind == 0):
    prv = []
  else:
    prv = works_list[cur_work_ind-1].excerpt
  
  if len(works_list) > cur_work_ind + 1:
    nxt = works_list[cur_work_ind+1].excerpt
  else:
    nxt = []
  return (prv, nxt)
  
 
@login_required
def index(request):
  all_details = Workdetail.objects.filter(user = request.user, excerpt__segs__user = request.user)
  work_toannotation = all_details.filter(Q(work_status = Work_status.ASSIGNED) | Q(work_status = Work_status.REASSIGNED)).annotate(t=Min('excerpt__text__text_name'), e_id=Min('excerpt__xml_id'), mod_date=Max('excerpt__segs__seg_change')).order_by('-work_status', 'mod_date')
  work_waiting = all_details.filter(work_status = Work_status.WAITING).annotate(mod_date=Max('excerpt__segs__seg_change')).order_by('mod_date')
  work_referee = all_details.filter(Q(work_status = Work_status.TO_REFEREE) | Q(work_status = Work_status.REWAITING)).annotate(mod_date=Max('excerpt__segs__seg_change')).order_by('-work_status', 'mod_date')
  work_done = all_details.filter(Q(work_status = Work_status.DONE) | Q(work_status = Work_status.REF_DONE)).annotate(mod_date=Max('excerpt__segs__seg_change')).order_by('-mod_date')
  

  context = {
    'work_toannotation': work_toannotation,
    'work_waiting':  work_waiting,
    'work_referee':  work_referee,
    'work_done':  work_done, 
    'status': WORK_STATUS_CHOICES,
    'exc_max': EXC_MAX,
    'exc_set': EXC_SET
  }
  return render(request, 'annotation/index.html', context)

@transaction.atomic
def index_post(request):
  dataJSON = json.loads(request.body)
  action = dataJSON.get('action', None)
  resp = json.dumps('OK')
  
  #Jeśli na moment trzeba zablokować pobieranie nowych próbek odkomentuj poniższą linię
  #return HttpResponse(json.dumps([]))
  
  exc_annot = Excerpt.objects.annotate(work_count=Count('details'))
  
  if REF_MODE:
    part_assigned = exc_annot.filter(work_count=1).filter(details__work_status__lt = Work_status.TO_REFEREE).exclude(details__user = request.user)
    not_assigned = exc_annot.filter(work_count=0)
    
    part_len = len(part_assigned)
    if part_len >= EXC_SET: # jest wystarczająco dużo próbek częściowo przypisanych - wybierz tylko z nich
      new_exc_part = random.sample(part_assigned, EXC_SET)
      new_exc = []
    else:
      new_exc_part = part_assigned # jest mniej próbek częściowo przypisanych niż potrzeba - weź je wszystkie i dobierz z tych nieprzypisanych
      new_nr = EXC_SET - part_len
      if len(not_assigned) >= new_nr:
        new_exc = random.sample(not_assigned, new_nr)
      else:
        new_exc = not_assigned
    
    # obsługa próbek częściowo przypisanych - usunęłam. W obu przypadkach tworzę kopię rekordów - ta z user=None zostaje jako wersja dla Superanotatora
    #for exc in new_exc_part:
      #new_wd = Workdetail(excerpt = exc, work_status = Work_status.ASSIGNED, user = request.user)
      #new_wd.save()
      ## W przypadku przypisania do jednego anotatora druga kopia segmentów danej próbki już istnieje i ma user=None
      #seg_list = Segment.objects.filter(excerpt = exc, user__isnull = True)
      #for seg in seg_list:
        #seg.user = request.user
        #seg.seg_change = timezone.now()
        #seg.save()
        
    new_excs = list(new_exc) + list(new_exc_part)
    for exc in new_excs:
      new_wd = Workdetail(excerpt = exc, work_status = Work_status.ASSIGNED, user = request.user)
      new_wd.save()
      # Pobierz listę segmentów próbki na podstawie pustego użytkownika i stwórz ich kopie dla nowego
      seg_list = Segment.objects.filter(excerpt = exc, user__isnull = True)
      for seg in seg_list:
        # zapisz kopię segmentu i nadaj jej użytkownika - oryginalna pozostanie z user=None
        seg.pk = None
        seg.user = request.user
        seg.seg_change = timezone.now()
        seg.save()  
        
  else:  # not in REF_MODE
    not_assigned = exc_annot.filter(work_count=0)
    new_exc_part = []
    if len(not_assigned) < EXC_SET:
      new_exc = not_assigned
    else:
      new_exc = random.sample(not_assigned, EXC_SET)
    
    for exc in new_exc:
      new_wd = Workdetail(excerpt = exc, work_status = Work_status.ASSIGNED, user = request.user)
      new_wd.save()
      # Przypisz segmentom nowego usera
      seg_list = Segment.objects.filter(excerpt = exc, user__isnull = True)
      for seg in seg_list:
        # pozostawiam kopię z user=None dla ułatwienia przejścia w tryb z dwoma anotatorami
        # gdy jako dane drugiego zostaną np. załadowane dane z tagera
        seg.pk = None
        seg.user = request.user
        seg.seg_change = timezone.now()
        seg.save()
    
  if (len(new_exc_part) + len(new_exc) == 0): # nie ma w bazie segmentów do przypisania - zwróć pustą listę
    resp = json.dumps([])
  else:
    wd_list = Workdetail.objects.filter(user = request.user, excerpt__segs__user = request.user).filter(Q(work_status = Work_status.ASSIGNED) | Q(work_status = Work_status.REASSIGNED)).annotate(t=Min('excerpt__text__text_name'), e_id=Min('excerpt__xml_id'), mod_date=Max('excerpt__segs__seg_change')).order_by('-work_status', 'mod_date')
    resp = jsonify_a(wd_list)
  
  return HttpResponse(resp)


@transaction.atomic
def excerpt_post(request, exc_id):
  dataJSON = json.loads(request.body)
  action = dataJSON.get('action', None)
  resp = json.dumps('OK')
  if action == 'setSentEnd':
    val = dataJSON['sent_end']
    for ident in dataJSON['seg_id']:
      segm = Segment.objects.get(pk=ident)
      segm.sent_end = val
      segm.seg_change = timezone.now()
      segm.save()
  elif action == 'setRejected':
    for elem in dataJSON['data']:
      segm = Segment.objects.get(pk=elem['seg_id'])
      segm.is_rejected = elem['reject']
      segm.seg_change = timezone.now()
      segm.save()
  elif action == 'setRejectedRef':
    for elem in dataJSON['data']:
      segm = Segment.objects.get(pk=elem['seg_id'])
      segm.ref_rejected = elem['reject']
      segm.seg_change = timezone.now()
      segm.save()
  elif action == 'setInterp':
    segm = Segment.objects.get(pk=dataJSON['seg_id'])
    segm.base = dataJSON['base']
    segm.ctag = dataJSON['ctag']
    segm.msd = dataJSON['msd']
    segm.radio_id = dataJSON['radio_id']
    segm.manual = dataJSON['manual']
    segm.interps = dataJSON['interps']
    segm.seg_change = timezone.now()
    segm.save()
  elif action == 'setTranskr':
    segm = Segment.objects.get(pk=dataJSON['seg_id'])
    segm.orth = dataJSON['orth']
    segm.orth_manual = True
    segm.seg_change = timezone.now()
    segm.save()
  elif action == 'sentCut':
    for elem in dataJSON['data']:
      segm = Segment.objects.get(pk=elem)
      segm.is_hidden = not(segm.is_hidden)
      segm.seg_change = timezone.now()
      segm.save()

  elif action == 'saveNotes':
    exc = Workdetail.objects.get(excerpt_id=dataJSON['exc_id'], user=request.user)
    exc.notes = dataJSON['notes']
    exc.save()

  elif action == 'endWork':
    exc_segs = Segment.objects.filter(excerpt = exc_id, user = request.user, is_hidden = False)
    seg_interps = exc_segs.exclude(is_rejected = Reject_status.YES).exclude(interps = '')
    if len(exc_segs) == 0:
      resp = json.dumps({'resp_code': 'NOT_OK', 'resp_message': 'Wszystkie segmenty zostały ukryte. Nie można zatwierdzić takiej próbki.'})
    elif len(exc_segs.filter(is_rejected = Reject_status.UNSET, choice_id__isnull = False)) > 0:
      resp = json.dumps({'resp_code': 'NOT_OK', 'resp_message': 'Nie zostały ujednoznacznione wszystkie warianty segmentacyjne.'})
    elif len(exc_segs.exclude(is_rejected = Reject_status.YES).filter(Q(base = '') & ~Q(interps = ''))) > 0:
      resp = json.dumps({'resp_code': 'NOT_OK', 'resp_message': 'Nie zostały ustawione interpretacje wszystkich segmentów.'})
    elif 1.0 * len(seg_interps.filter(ctag = 'ignndm')) / len(seg_interps) > 1.0 * MAX_IGNNDM / 100:
      resp = json.dumps({'resp_code': 'NOT_OK', 'resp_message': 'Liczba segmentów oznaczonych jako "ignndm" przekracza ' + str(MAX_IGNNDM) + '%.'})
    elif 1.0 * len(seg_interps.filter(ctag = 'ign')) / len(seg_interps) > 1.0 * MAX_IGN / 100:
      resp = json.dumps({'resp_code': 'NOT_OK', 'resp_message': 'Liczba segmentów oznaczonych jako "ign" przekracza ' + str(MAX_IGN) + '%.'})      
    else:
      exc_work = Workdetail.objects.filter(excerpt_id=exc_id)
      my_work_det = exc_work.get(user = request.user)
      if REF_MODE:
        other_work = exc_work.filter(~Q(user = request.user))
        if len(other_work):
          other_work_det = other_work[0]
          if my_work_det.work_status > Work_status.REWAITING: # Może się zdarzyć, jeśli wystąpił błąd typu time-out
            resp = json.dumps({'resp_code': 'OK', 'resp_message': 'Próbka została już sprawdzona.'})
          elif other_work_det.work_status > my_work_det.work_status:  # Druga wersja już jest w stanie oczekiwania
            resp = json.dumps({'resp_code': 'OK', 'resp_message': checkExcerpts(exc_id, my_work_det, other_work_det)})
          else:
            resp = json.dumps({'resp_code': 'OK', 'resp_message': setWait(my_work_det)})
        else:
          resp = json.dumps({'resp_code': 'OK', 'resp_message': setWait(my_work_det)})

      else: # not in REF_MODE
        my_work_det.work_status = Work_status.DONE
        my_work_det.status_change = timezone.now()
        my_work_det.save()
        resp = json.dumps({'resp_code': 'OK', 'resp_message': 'Próbka została zakończona.'})

  elif action == 'endWorkRef':
    exc_segs = Segment.objects.filter(excerpt = exc_id).exclude(ref_id = 0)
    if len(exc_segs.filter(ref_rejected = Reject_status.UNSET)) > 0:
      resp = json.dumps({'resp_code': 'NOT_OK', 'resp_message': 'Nie zostały ustalone warianty anotacyjne wszystkich segmentów.'})
    else:
      segs_nothidden = exc_segs.filter(ref_rejected = Reject_status.NO, is_hidden = False)
      seg_interps = segs_nothidden.exclude(is_rejected = Reject_status.YES).exclude(interps = '')
      if len(segs_nothidden) == 0:
        resp = json.dumps({'resp_code': 'NOT_OK', 'resp_message': 'Wszystkie segmenty zostały ukryte. Nie można zatwierdzić takiej próbki.'})
      elif len(segs_nothidden.filter(is_rejected = Reject_status.UNSET, choice_id__isnull = False)) > 0:
        resp = json.dumps({'resp_code': 'NOT_OK', 'resp_message': 'Nie zostały ujednoznacznione wszystkie warianty segmentacyjne.'})
      elif len(segs_nothidden.exclude(is_rejected = Reject_status.YES).filter(Q(base = '') & ~Q(interps = ''))) > 0:
        resp = json.dumps({'resp_code': 'NOT_OK', 'resp_message': 'Nie zostały ustawione interpretacje wszystkich segmentów.'})
      elif 1.0 * len(seg_interps.filter(ctag = 'ignndm')) / len(seg_interps) > 1.0 * MAX_IGNNDM / 100:
        resp = json.dumps({'resp_code': 'NOT_OK', 'resp_message': 'Liczba segmentów oznaczonych jako "ignndm" przekracza ' + str(MAX_IGNNDM) + '%.'})
      elif 1.0 * len(seg_interps.filter(ctag = 'ign')) / len(seg_interps) > 1.0 * MAX_IGN / 100:
        resp = json.dumps({'resp_code': 'NOT_OK', 'resp_message': 'Liczba segmentów oznaczonych jako "ign" przekracza ' + str(MAX_IGN) + '%.'})      
      else:
        exc_work = Workdetail.objects.filter(excerpt_id=exc_id)
        for w in exc_work:
          w.work_status = Work_status.REF_DONE
          w.status_change = timezone.now()
          w.save()
        resp = json.dumps({'resp_code': 'OK', 'resp_message': 'Próbka zakończona.'})

  elif action == 'segDivide':
    s_list = []
    segm = Segment.objects.get(pk=dataJSON['seg_id'])
    segm_div_choice = segm.choice_id
    segm_choice_list = Segment.objects.filter(excerpt = segm.excerpt, user = segm.user_id, xml_id = segm.xml_id).order_by('choice_id', 'paren_id')
    choice_nr = segm_choice_list.last().choice_id or 1
    p_id = 1
    for segm_choice in segm_choice_list:
      if segm_choice.choice_id is None:
        segm_choice.choice_id = 1
      segm_choice.is_rejected = Reject_status.YES
      segm.seg_change = timezone.now()
      segm_choice.save()
      if (segm_div_choice is None) or (segm_choice.choice_id == segm_div_choice):
        if (segm.paren_id is None) or (segm_choice.paren_id == segm.paren_id):
          translit_list = dataJSON['translit_l']
          orth_list = dataJSON['orth_l']
          l = 0
          for i, elem in enumerate(translit_list):
            s = Segment(excerpt = segm.excerpt, user = segm.user, xml_id = segm.xml_id, translit = elem, orth = orth_list[i], sent_end = False, seg_type = segm.seg_type, corresp_elem = segm.corresp_elem, corresp_pos = segm.corresp_pos + l, corresp_len = len(elem), choice_id = choice_nr + 1, paren_id = p_id, korbeusz = False, seg_change = timezone.now(), ref_id = segm.ref_id, ref_rejected = segm.ref_rejected)
            lex = {'base': orth_list[i], 'ctag': 'ign', 'msd': [{'manual': False, 'val': ''}]}
            s.interps = json.dumps([lex])
            if i == 0: # na pierwszym segmencie ustaw spację przed
              s.nps = segm.nps
            else:
              s.nps = True
            if i == (len(elem) - 1): # na ostatnim segmencie ustaw koniec zdania
              s.sent_end = segm.sent_end
            else:
              s.sent_end = False
            s.is_rejected = Reject_status.NO
            s.save()
            l += len(elem)
            p_id += 1
        else:
          s = segm_choice
          s.pk = None # to będzie kopia rekordu segm_choice
          s.paren_id = p_id
          s.choice_id = choice_nr + 1
          s.seg_change = timezone.now()
          s.is_rejected = Reject_status.NO
          s.save()
          p_id += 1
    if dataJSON['mode'] == 'referee':
      resp = jsonify(Segment.objects.filter(excerpt = segm.excerpt).exclude(ref_id = 0).order_by('ref_id', 'user', 'xml_id', 'choice_id', 'paren_id'))
    else:
      resp = jsonify(Segment.objects.filter(excerpt = segm.excerpt, user = request.user).order_by('xml_id', 'choice_id', 'paren_id'))
  elif action == 'segConcat':
    xml_id1 = dataJSON['word1_id']
    xml_id2 = dataJSON['word2_id']
    ref1 = dataJSON['word1_ref']
    ref2 = dataJSON['word2_ref']
    ch_id1 = dataJSON['word1_choice'] or 1
    ch_id2 = dataJSON['word2_choice'] or 1
    word1_list = list(Segment.objects.filter(excerpt = dataJSON['exc_id'], user = dataJSON['user'], xml_id = xml_id1).order_by('choice_id', 'paren_id'))
    word2_list = list(Segment.objects.filter(excerpt = dataJSON['exc_id'], user = dataJSON['user'], xml_id = xml_id2).order_by('choice_id', 'paren_id'))
    
    word1_arr = [] # zawiera listy odpowiadające kolejnym choice
    word2_arr = [] # jw
    choice = -1
    for elem in word1_list:
      ch = elem.choice_id or 1
      if ch == choice:
        word1_arr[-1].append(elem)
      else:
        word1_arr.append([elem])
        choice = ch
    choice = -1
    for elem in word2_list:
      ch = elem.choice_id or 1
      if ch == choice:
        word2_arr[-1].append(elem)
      else:
        word2_arr.append([elem])
        choice = ch
    # wygeneruj wszystkie kombinacje dotychczasowych choice'ów
    # wykorzystując istniejące rekordy + dodaj, gdzie trzeba
    ch = 1
    del_list = []
    if ref1 == ref2:
      change_ref = None
    else:
      change_ref = ref1
    for i, w1 in enumerate(word1_arr, start=1):
      paren = 1
      for j, w2 in enumerate(word2_arr, start=1):
        if i == j: # zaktualizuj w1 i w2
          paren = wordSet('change', w1, None, None, ch, paren, Reject_status.YES)
          paren = wordSet('change', w2, xml_id1, change_ref, ch, paren, Reject_status.YES)
        elif (j > len(word1_arr)) and (i == 1): # skopiuj w1, zaktualizuj w2
          paren = wordSet('add', w1, None, None, ch, paren, Reject_status.YES)
          paren = wordSet('change', w2, xml_id1, change_ref, ch, paren, Reject_status.YES)
        elif (i > len(word2_arr)) and (j == 1): # zaktualizuj w1, skopiuj w2
          paren = wordSet('change', w1, None, None, ch, paren, Reject_status.YES)
          paren = wordSet('add', w2, xml_id1, change_ref, ch, paren, Reject_status.YES)
        else: # skopiuj w1 i w2
          paren = wordSet('add', w1, None, None, ch, paren, Reject_status.YES)
          paren = wordSet('add', w2, xml_id1, change_ref, ch, paren, Reject_status.YES)
        paren = 1
        ch += 1

    # wygeneruj nowy choice z połączonymi segmentami
    paren = 1
    # skopiuj segmenty z w1 poza ostatnim
    paren = wordSet('add', word1_arr[ch_id1-1][:-1], None, None, ch, paren, Reject_status.NO)

    # s1 to ostatni segment z w1, s2 to pierwszy segment z w2 - połącz je w jeden nowy
    s1 = word1_arr[ch_id1-1][-1]
    s2 = word2_arr[ch_id2-1][0]
    # Nowy rekord budowany jest na bazie s1
    s1.pk = None
    s1.korbeusz = False
    if s2.sent_end:
      s1.sent_end = True
    if s1.seg_type == '': # !!!Nie można dopuścić różnych typów!!!
      if s2.seg_type != '':
        s1.seg_type = s2.seg_type
    else:
      if s2.seg_type != '':
        s1.seg_type = '{}, {}'.format(s1.seg_type, s2.seg_type)
    if s2.nps:
      sp = ''
    else:
      sp = ' '
    s1.translit = s1.translit + sp + s2.translit
    s1.orth = s1.orth + sp + s2.orth
    s1.base = ''
    s1.ctag = ''
    s1.msd = ''
    s1.radio_id = ''
    lex = {'base': s1.orth, 'ctag': 'ign', 'msd': [{'manual': False, 'val': ''}]}
    s1.interps = json.dumps([lex])
    if s1.corresp_elem != s2.corresp_elem: # !!!Nie można dopuścić różnych coresp_elem!!!
      s1.corresp_elem = s1.corresp_elem + ', ' + s2.corresp_elem
    s1.corresp_len = len(s1.translit)
    s1.choice_id = ch
    if (dataJSON['word1_len'] + dataJSON['word2_len'] == 2):
      s1.paren_id = None
    else:
      s1.paren_id = paren
    s1.seg_change = timezone.now()
    s1.is_rejected = Reject_status.NO
    s1.save()    
    paren += 1
    
    # skopiuj segmenty z w2 poza pierwszym
    paren = wordSet('add', word2_arr[ch_id2-1][1:], xml_id1, change_ref, ch, paren, Reject_status.NO)
    
    # jeśli trzeba było aktualizować ref_id, to zmień je we wszystkich pozostałych segmentach (innych anotatorów)
    if change_ref:
      ref_list = Segment.objects.filter(excerpt = dataJSON['exc_id'], ref_id = ref2)
      for s in ref_list:
        s.ref_id = ref1
        s.seg_change = timezone.now()
        s.save()
    
    if dataJSON['mode'] == 'referee':
      resp = jsonify(Segment.objects.filter(excerpt = dataJSON['exc_id']).exclude(ref_id = 0).order_by('ref_id', 'user', 'xml_id', 'choice_id', 'paren_id'))
    else:
      resp = jsonify(Segment.objects.filter(excerpt = dataJSON['exc_id'], user = request.user).order_by('xml_id', 'choice_id', 'paren_id'))

  elif action == 'segConcatChoice':
    xml_id1 = dataJSON['word1_id']
    ch_id1 = dataJSON['word1_choice']
    paren_id1 = dataJSON['word1_paren']
    choice_qs = Segment.objects.filter(excerpt = dataJSON['exc_id'], user = dataJSON['user'], xml_id = xml_id1).order_by('choice_id', 'paren_id')
    choice_list = list(choice_qs)
    choice_max = choice_list[-1].choice_id
    i = 1 #nowy paren_id
    for elem in choice_list:
      elem.is_rejected = Reject_status.YES
      elem.seg_change = timezone.now()
      elem.save()
      if (elem.choice_id == ch_id1):
        if (elem.paren_id == paren_id1):
          ind = choice_list.index(elem)
          elem_next = choice_list[ind + 1]
          elem.pk = None
          elem.korbeusz = False
          elem.choice_id = choice_max + 1
          if (dataJSON['word1_len'] == 2):
            elem.paren_id = None
          else:
            elem.paren_id = i
          if elem_next.nps:
            sp = ''
          else:
            sp = ' '
          elem.translit = elem.translit + sp + elem_next.translit
          elem.orth = elem.orth + sp + elem_next.orth
          if elem_next.sent_end:
            elem.sent_end = True
          elem.base = ''
          elem.ctag = ''
          elem.msd = ''
          elem.radio_id = ''
          lex = {'base': elem.orth, 'ctag': 'ign', 'msd': [{'manual': False, 'val': ''}]}
          elem.interps = json.dumps([lex])
          elem.corresp_len = len(elem.translit)
          elem.is_rejected = Reject_status.NO
          elem.seg_change = timezone.now()
          elem.save()
          i += 1
        elif (elem.paren_id != paren_id1 + 1):
          elem.pk = None
          elem.choice_id = choice_max + 1
          elem.paren_id = i
          elem.is_rejected = Reject_status.NO
          elem.seg_change = timezone.now()
          elem.save()
          i += 1
    if dataJSON['mode'] == 'referee':
      resp = jsonify(Segment.objects.filter(excerpt = dataJSON['exc_id']).exclude(ref_id = 0).order_by('ref_id', 'user', 'xml_id', 'choice_id', 'paren_id'))
    else:
      resp = jsonify(Segment.objects.filter(excerpt = dataJSON['exc_id'], user = request.user).order_by('xml_id', 'choice_id', 'paren_id'))
   
  return HttpResponse(resp)

def wordSet(mode, word_array, new_id, new_ref, choice, paren, reject):
  #if mode == 'change':
  #print('{}, {}, {}, {}, {}, {}'.format(mode, word_array, new_id, choice, paren, reject))
  for elem in word_array:
    if mode == 'add':
      s = deepcopy(elem)
      s.pk = None
    else:
      s = elem
    if new_id is not None:
      s.xml_id = new_id
    if new_ref is not None:
      s.ref_id = new_ref
    s.choice_id = choice
    s.paren_id = paren
    s.is_rejected = reject
    s.seg_change = timezone.now()
    s.save()
    paren += 1
  return paren

def setWait(workdet):
  workdet.work_status += 1
  workdet.status_change = timezone.now()
  workdet.save()
  if workdet.work_status == Work_status.WAITING:
    return "Drugi anotator jeszcze nie zakończył prac nad swoją wersją. Próbka wchodzi w stan oczekiwania."
  else:
    return "Drugi anotator jeszcze nie zakończył przeglądać próbki po konflikcie. Próbka wchodzi w stan oczekiwania."

# Zwraca ostatni identyfikator, na którym co najmniej jeden z segmentów jest ukryty.
# Potem trzeba jeszcze je wyrównać w sensie pozycji w tekście źródłowym. 
def alignHidden(exc_user1, exc_user2):
  end1 = 0
  end2 = 0
  
  for seg in exc_user1:
    if seg.is_hidden:
      end1 = seg.xml_id
    else:
      break
  
  for seg in exc_user2:
    if seg.is_hidden:
      end2 = seg.xml_id
    else:
      break
  
  return max(end1, end2)

# Znajduje wyrównany ciąg segmentów od xml_id='seg_start' do 'seg_end' u obu anotatorów. Odpowiednie listy segmentów zwraca
# na 'seglist1' oraz 'seglist2'. Na exc_user1 i exc_user2 listy segmentów dla całej próbki u obu anotatorów. 
def alignExcerpts(exc_user1, exc_user2, seg_start, seg_end, check_hidden):
  i = seg_start
  j = seg_end
  hidden_start = False
  if (seg_start == 1) and check_hidden:
    ind_hidden = alignHidden(exc_user1, exc_user2)
    if ind_hidden:
      seglist1 = list(exc_user1.filter(xml_id__lte=ind_hidden))
      seglist2 = list(exc_user2.filter(xml_id__lte=ind_hidden))
      i = ind_hidden
      hidden_start = True
    else:
      seglist1 = list(exc_user1.filter(xml_id=seg_start))
      seglist2 = list(exc_user2.filter(xml_id=seg_start))
  else:
    seglist1 = list(exc_user1.filter(xml_id=seg_start))
    seglist2 = list(exc_user2.filter(xml_id=seg_start))

  # jeśli trzeba dołączaj kolejne segmenty z kolejnych xml_id aż na ostatnim zrówna się pozycja w oryginalnym tekście
  # jeśli którykolwiek z segmentów jest ukryty, to rozszerz o niego wyrównany fragment jeśli check_hidden = True
    
  while (i <= j):
    if (len(seglist1) == 0) or (len(seglist2) == 0):  # puste nie powinny się zdarzyć na początku, bo listy są nimi dopełniane na końcu, ale...
      i += 1
      seglist1.extend(list(exc_user1.filter(xml_id=i)))
      seglist2.extend(list(exc_user2.filter(xml_id=i)))
    else:
      if check_hidden and (not hidden_start): #jeśli na którejkolwiek z list pojawił się ukryty segment, to zwróć całą resztę jako wyrównany fragment (ukryty koniec)
        if seglist1[-1].is_hidden or seglist2[-1].is_hidden:
          seglist1.extend(list(exc_user1.filter(xml_id__gt=i)))
          seglist2.extend(list(exc_user2.filter(xml_id__gt=i)))
          return (seglist1, seglist2, seg_start, j)
      if (seglist1[-1].corresp_pos is None) and (seglist2[-1].corresp_pos is None): #trafiło na pusty element w obu listach - wcześniej musiało być wyrównanie więc segmenty są OK
        break
      else:
        if ((seglist1[-1].corresp_pos + seglist1[-1].corresp_len) != (seglist2[-1].corresp_pos + seglist2[-1].corresp_len)):
          if hidden_start:
            hidden_start = False
          i += 1
          seglist1.extend(list(exc_user1.filter(xml_id=i)))
          seglist2.extend(list(exc_user2.filter(xml_id=i)))
        else:
          break
  seg_end = i
  # Na indeksach <seg_start, seg_end> jest wyrównany fragment. Sprawdź, czy nie ma jakichś pustych xml_id dalej 
  # w obu ciągach segmentów i ewentualnie zwiększ o nie 'end' (to pozostałości po złączonych segmentach)
  i += 1
  while (i <= j):
    test1 = exc_user1.filter(xml_id=i)
    test2 = exc_user2.filter(xml_id=i)
    if (len(test1) == 0) and (len(test2) == 0):
      seg_end = i
      i += 1
    else:
      break
  return (seglist1, seglist2, seg_start, seg_end)


# Funkcja wywoływana dla workdet1 jeśli trzeba porównać z workdet2, która jest w trybie oczekującym
def checkExcerpts(exc_id, workdet1, workdet2):
  conflict = False
  # Pobierz segmenty anotatorów z wyłączeniem odrzuconych wariantów segmentacyjnych
  seg_user1 = Segment.objects.filter(excerpt = exc_id, user = workdet1.user).exclude(is_rejected = Reject_status.YES).order_by('xml_id', 'choice_id', 'paren_id')
  seg_user2 = Segment.objects.filter(excerpt = exc_id, user = workdet2.user).exclude(is_rejected = Reject_status.YES).order_by('xml_id', 'choice_id', 'paren_id')
  j = max([seg_user1.last().xml_id, seg_user2.last().xml_id])
  # na i kolejne wartości xml_id
  i = 1
  t = time.clock()
  while i <= j:
    
    seglist1, seglist2, exc_start, exc_end = alignExcerpts(seg_user1, seg_user2, i, j, False)

    # przejdź po listach segmentów obu anotatorów sprawdzając ich zgodność
    ind1 = 0
    ind2 = 0
    while (ind1 < len(seglist1)) and (ind2 < len(seglist2)):
      if (seglist1[ind1].corresp_pos == seglist2[ind2].corresp_pos) and (seglist1[ind1].corresp_len == seglist2[ind2].corresp_len): # segmenty odpowiadają temu samemu fragmentowi oryginalnego tekstu
        if (seglist1[ind1].is_hidden == True) and (seglist2[ind2].is_hidden == True):
          seglist1[ind1].is_conflict = False
          seglist2[ind2].is_conflict = False
        elif (seglist1[ind1].is_hidden == seglist2[ind2].is_hidden) and (seglist1[ind1].corresp_pos is None) and (seglist2[ind2].corresp_pos is None): # puste elementy, oba odkryte - bez interpretacji a więc i bez konfliktu
          seglist1[ind1].is_conflict = False
          seglist2[ind2].is_conflict = False          
        elif (seglist1[ind1].is_hidden == seglist2[ind2].is_hidden) and ((seglist1[ind1].ctag in ['ign', 'ignndm']) or (seglist2[ind2].ctag in ['ign', 'ignndm'])): # ign i ignndm oznacz jako konflikty - do decyzji superanotatora
          conflict = True
          seglist1[ind1].is_conflict = True
          seglist2[ind2].is_conflict = True          
        elif (seglist1[ind1].is_hidden == seglist2[ind2].is_hidden) and (seglist1[ind1].translit == seglist2[ind2].translit) and (seglist1[ind1].orth == seglist2[ind2].orth) and (seglist1[ind1].base == seglist2[ind2].base) and (seglist1[ind1].ctag == seglist2[ind2].ctag) and (seglist1[ind1].msd == seglist2[ind2].msd) and (seglist1[ind1].sent_end == seglist2[ind2].sent_end):
          seglist1[ind1].is_conflict = False
          seglist2[ind2].is_conflict = False
        else:
          conflict = True
          seglist1[ind1].is_conflict = True
          seglist2[ind2].is_conflict = True
        seglist1[ind1].save()
        seglist2[ind2].save()        
      else: # nie ma zgodności segmentów z oryginalnym tekstem - dojedź do wspólnej pozycji zakończenia
        #print(u'Niezgodność na segmentach: {}, {}, {} oraz {}, {}, {}'.format(unicode(seglist1[ind1].translit), seglist1[ind1].corresp_pos, seglist1[ind1].corresp_len, unicode(seglist2[ind2].translit), seglist2[ind2].corresp_pos, seglist2[ind2].corresp_len))
        conflict = True
        seglist1[ind1].is_conflict = True
        seglist1[ind1].save()
        seglist2[ind2].is_conflict = True
        seglist2[ind2].save()
        pos1 = seglist1[ind1].corresp_pos + seglist1[ind1].corresp_len
        pos2 = seglist2[ind2].corresp_pos + seglist2[ind2].corresp_len
        while (ind1 < len(seglist1)) and (ind2 < len(seglist2)) and (pos1 != pos2):
          if (pos1 > pos2):
            #print(u'Biorę następny ind2: {}'.format(ind2+1))
            ind2 += 1
            pos2 = seglist2[ind2].corresp_pos + seglist2[ind2].corresp_len
            seglist2[ind2].is_conflict = True
            seglist2[ind2].save()
          else:
            #print(u'Biorę następny ind1: {}'.format(ind1+1))
            ind1 += 1
            pos1 = seglist1[ind1].corresp_pos + seglist1[ind1].corresp_len
            seglist1[ind1].is_conflict = True
            seglist1[ind1].save()
      ind1 += 1
      ind2 += 1
        
#    i += 1
    i = exc_end + 1

#  print "Czas szukania wspólnych segmentów: {}".format(time.clock() - t)


  if (workdet1.work_status == Work_status.ASSIGNED):
    if conflict:
      if workdet1.user.username == 'tager':  # W przypadku wersji tagera od razu przechodzi w stan REWAITING
        workdet1.work_status = Work_status.REWAITING
      else:
        workdet1.work_status = Work_status.REASSIGNED
      if workdet2.user.username == 'tager': # W przypadku wersji tagera od razu przechodzi w stan REWAITING
        workdet2.work_status = Work_status.REWAITING
      else:
        workdet2.work_status = Work_status.REASSIGNED
      out_message = "Wystąpiły konflikty z drugim anotatorem. Próbka została zwrócona do ponownego przejrzenia i ewentualnej poprawki."
    else:
      workdet1.work_status = Work_status.DONE
      workdet2.work_status = Work_status.DONE
      out_message = "Próbka zgodna z próbką drugiego anotatora i praca nad nią zostaje zakończona.\nDobra robota!"
  else: # czyli musiał być REASSIGNED
    if conflict:
      workdet1.work_status = Work_status.TO_REFEREE
      workdet2.work_status = Work_status.TO_REFEREE
      out_message = "Niestety, za drugim razem również wystąpiły konflikty z drugim anotatorem. Próbka zostaje przekazana do Superanotatora."
    else:
      workdet1.work_status = Work_status.DONE
      workdet2.work_status = Work_status.DONE
      out_message = "Drugie podejście pomogło. Próbka zgodna z próbką drugiego anotatora i praca nad nią zostaje zakończona.\nDobra robota!"
  workdet1.status_change = timezone.now()
  workdet2.status_change = timezone.now()
  workdet1.save()
  workdet2.save()
  return out_message
  
  
@login_required
def excerpt(request, exc_id):
  exc = get_object_or_404(Excerpt, pk = exc_id)
  if request.method == 'GET':
    wd = Workdetail.objects.get(excerpt = exc, user = request.user)
    prv, nxt = getPrevNextExc(exc, request.user)
    
    exc_st = exc.details.filter(user = request.user)[0].work_status
    if exc_st in [Work_status.ASSIGNED, Work_status.REASSIGNED]:
      can_edit = 1
    else:
      can_edit = 0

    if exc_st == Work_status.REF_DONE: # widok skończonej próbki dla anotatora po superanotacji
      segs = exc.segs.exclude(ref_id = 0).filter(ref_rejected = Reject_status.NO).order_by('ref_id', 'xml_id', 'choice_id', 'paren_id')
    else:
      segs = exc.segs.all().filter(user = request.user).order_by('xml_id', 'choice_id', 'paren_id')
    
    context = {
      'exc': exc,
      'can_edit': can_edit,
      'mode': 'annotator',
      'next_exc': nxt,
      'prev_exc': prv,
      'segs': segs,
      'notes': wd.notes
    }
    return render(request, 'annotation/excerpt.html', context)
#  elif request.method == 'POST':

@login_required
def stats(request):
  all_users = User.objects.annotate(work_count=Count('workdetail'))
  all_refs = User.objects.annotate(ref_count=Count('referee'))
  work_users = all_users.filter(work_count__gt=0).order_by('last_name', 'first_name')
  ref_users = all_refs.filter(ref_count__gt=0).order_by('last_name', 'first_name')

  work_stats = []
  work_sum = { 's1': 0,
  's2': 0,
  's3': 0,
  's4': 0,
  's5': 0,
  's6': 0,
  's7': 0,
  'p': 0,
  'np': 0}
  
  for u in work_users:
    wd = Workdetail.objects.filter(user = u)
    wsu = {'user': u, 
    'status1': wd.filter(work_status = 1).count(), 
    'status2': wd.filter(work_status = 2).count(), 
    'status3': wd.filter(work_status = 3).count(), 
    'status4': wd.filter(work_status = 4).count(),
    'status5': wd.filter(work_status = 5).count(),
    'status6': wd.filter(work_status = 6).count(),
    'status7': wd.filter(work_status = 7).count(),
    'payed': wd.filter(pay_date__isnull = False).count(),
    'not_payed': wd.filter(pay_date__isnull = True, work_status__gt = 3).count()
    }
    
    work_stats.append(wsu)
    
    work_sum['s1'] += wsu['status1']
    work_sum['s2'] += wsu['status2']
    work_sum['s3'] += wsu['status3']
    work_sum['s4'] += wsu['status4']
    work_sum['s5'] += wsu['status5']
    work_sum['s6'] += wsu['status6']
    work_sum['s7'] += wsu['status7']
    work_sum['p'] += wsu['payed']
    work_sum['np'] += wsu['not_payed']
  
  ref_stats = []
  ref_sum = { 'a': 0,
  'd': 0,
  'p': 0,
  'np': 0}
  
  for r in ref_users:
    wd = Workdetail.objects.filter(ref_user = r).values('excerpt', 'work_status', 'ref_user', 'ref_pay_date').distinct()
    wdref = {'user': r, 
    'assigned': wd.filter(work_status = 5).count(), 
    'done': wd.filter(work_status__gt = 5).count(), 
    'payed': wd.filter(ref_pay_date__isnull = False).count(),
    'not_payed': wd.filter(work_status__gt = 5, ref_pay_date__isnull = True).count()
    }
    
    ref_stats.append(wdref)
    
    ref_sum['a'] += wdref['assigned']
    ref_sum['d'] += wdref['done']
    ref_sum['p'] += wdref['payed']
    ref_sum['np'] += wdref['not_payed']
  
  exc_annot = Excerpt.objects.annotate(work_count=Count('details'))
  
  exc_stats = {'all': exc_annot.count(),
  'none': exc_annot.filter(work_count=0).count(),
  'partial': exc_annot.filter(work_count=1).count(),
  'two_annot': exc_annot.filter(work_count=2).count(),
  'done': exc_annot.filter(details__work_status__gt=5).values_list('id', flat=True).order_by('id').distinct().count()
  }

  context = {
    'work_stats': work_stats,
    'work_sum': work_sum,
    'ref_stats': ref_stats,
    'ref_sum': ref_sum,
    'exc_stats': exc_stats,
  }
  return render(request, 'annotation/stats.html', context)

@login_required
def superlist(request, page_nr=1):
  exc_annot = Excerpt.objects.annotate(
    work_count=Count('details'), 
    work_stat=Min('details__work_status'), 
    stat_change=Min('details__status_change'),
    sort_action=Case(
      When(details__ref_user=request.user, then=0), 
      When(details__ref_user=None, then=1), 
      default=2, 
      output_field=IntegerField(),
      ),
    )
  exc_super_all = exc_annot.filter(work_count = 2, work_stat = 5).order_by('sort_action', '-stat_change')

  paginator = Paginator(exc_super_all, 25)
  try:
    exc_super = paginator.page(page_nr)
  except PageNotAnInteger:
    exc_super = paginator.page(1)
  except EmptyPage:
    exc_super = paginator.page(paginator.num_pages)
  
  context = {'superlist': exc_super,
  }
  return render(request, 'annotation/superlist.html', context)

@login_required
def superview(request, exc_id):
  exc = get_object_or_404(Excerpt, pk = exc_id)
  exc_users = [det.user for det in Workdetail.objects.filter(excerpt = exc).order_by('user')]
  seg_user1 = Segment.objects.filter(excerpt_id = exc, user = exc_users[0]).exclude(is_rejected = Reject_status.YES).order_by('xml_id', 'choice_id', 'paren_id')
  seg_user2 = Segment.objects.filter(excerpt_id = exc, user = exc_users[1]).exclude(is_rejected = Reject_status.YES).order_by('xml_id', 'choice_id', 'paren_id')

  i = min(seg_user1.first().xml_id, seg_user2.first().xml_id)
  j = max(seg_user1.last().xml_id, seg_user2.last().xml_id)
  
  ''' Lista, w której na kolejnych pozycjach (odpowiadających kolejnym xml_id) znajdują się słowniki, w których:
  - col_num - liczba kolumn przy porównaniu (jesli == 1, to wersje są tożsame) 
  - u1 - wersja użytkownika 1 (jeśli zgodne, to tylko tę przekazuję)
  - u2 - wersja użytkownika 2
  '''
  segs = []
  
  t = time.clock()
  while (i<=j):
    seglist1, seglist2, exc_start, exc_end = alignExcerpts(seg_user1, seg_user2, i, j, False)
    if (len(seglist1) == 0) and (len(seglist2) == 0):  # nie powinno się zdarzyć, po puste są dołączane na końcu, ale...
      segs.append({'col_nr': 1, 'u1': []}) # zgodna interpretacja, pusta lista
    elif len(seglist1) == 0:
      segs.append(check_conflicts(seglist1, seglist2, 2))
    elif len(seglist2) == 0:
      segs.append(check_conflicts(seglist1, seglist2, 2))
    else:
      diff = False
      for s in seglist1: # jeśli na którymkolwiek podsegmencie na danej pozycji u pierwszego anotatora wersje się nie zgadzają, to nie zgadza się całe słowo
        if s.is_conflict:
          diff = True
          break
      if diff:
        segs.append(check_conflicts(seglist1, seglist2, 2))
      else:
        segs.append(check_conflicts(seglist1, [], 1))
    #print('segs: {}'.format(segs))
    i = exc_end + 1
    
#  print "Czas szukania wspólnych segmentów - superanotacja: {}".format(time.clock() - t)

  context = {'exc': exc,
  'exc_users': {'u1': exc_users[0], 'u2': exc_users[1]},
  'segs': segs
  }
  return render(request, 'annotation/superview.html', context)

def check_conflicts(s1, s2, col_nr):
  if col_nr == 2:
    ls1 = []
    ls2 = []
    base_dict = {'hidden_confl': True, 'translit_confl': True, 'orth_confl': True, 'base_confl': True, 'ctag_confl': True, 'msd_confl': True, 'sent_end_confl': True}
    if len(s1) == 0:
      for elem in s2:
        d1 = deepcopy(base_dict)
        d1['seg'] = elem
        ls2.append(d1)
    elif len(s2) == 0:
      for elem in s1:
        d1 = deepcopy(base_dict)
        d1['seg'] = elem
        ls1.append(d1)
    elif len(s1) != len(s2):
      for elem in s1:
        d1 = deepcopy(base_dict)
        d1['seg'] = elem
        ls1.append(d1)
      for elem in s2:
        d1 = deepcopy(base_dict)
        d1['seg'] = elem
        ls2.append(d1)
    else:
      for i, elem in enumerate(s1):
        d1 = deepcopy(base_dict)
        if elem.is_hidden == s2[i].is_hidden:
          d1['hidden_confl'] = False
        if elem.translit == s2[i].translit:
          d1['translit_confl'] = False
        if elem.orth == s2[i].orth:
          d1['orth_confl'] = False
        if elem.base == s2[i].base:
          d1['base_confl'] = False
        if elem.ctag == s2[i].ctag:
          d1['ctag_confl'] = False
        if elem.msd == s2[i].msd:
          d1['msd_confl'] = False
        if elem.sent_end == s2[i].sent_end:
          d1['sent_end_confl'] = False
        d2 = deepcopy(d1)
        d1['seg'] = elem
        d2['seg'] = s2[i]
        ls1.append(d1)
        ls2.append(d2)
    return {'col_nr': col_nr, 'u1': ls1, 'u2': ls2}
  else:
    ls1 = []
    #base_dict = {'hidden_confl': False, 'translit_confl': False, 'orth_confl': False, 'base_confl': False, 'ctag_confl': False, 'msd_confl': False, 'sent_end_confl': False}
    for elem in s1:
      d1 = {'seg': elem}
      ls1.append(d1)
    return {'col_nr': col_nr, 'u1': ls1}

@login_required
def superexcerpt(request, exc_id):
  exc = get_object_or_404(Excerpt, pk = exc_id)

  if exc.getReferee() is None: # przygotuj dane do superanotacji
#    t = time.clock()

    with transaction.atomic():
      worklist = Workdetail.objects.filter(excerpt = exc)
      for workdet in worklist:
        workdet.ref_user = request.user
        workdet.status_change = timezone.now()
        workdet.save()

      # Na podstawie wybranych wariantów obu anotatorów ustaw wartości ref_id
      exc_users = [det.user for det in worklist.filter(excerpt = exc).order_by('user')]
      seg_user1 = exc.segs.filter(user = exc_users[0]).exclude(is_rejected = Reject_status.YES).order_by('xml_id', 'choice_id', 'paren_id')
      seg_user2 = exc.segs.filter(user = exc_users[1]).exclude(is_rejected = Reject_status.YES).order_by('xml_id', 'choice_id', 'paren_id')
      
      i = 1
      j = max(seg_user1.last().xml_id, seg_user2.last().xml_id) # uwaga przy wypełnianiu dla superanotatora - mogą być jeszcze dalsze segmenty odrzucone u obu anotatorów
      ref = 1
      
      t = time.clock()
      while (i<=j):
        seglist1, seglist2, exc_start, exc_end = alignExcerpts(seg_user1, seg_user2, i, j, True)
        
        #sprawdź, czy wersje obu anotatorów są zgodne
        diff = False
#        print "{}, {}, {}, {}".format(seglist1, seglist2, exc_start, exc_end)
        if len(seglist1) == len(seglist2):
          for s in seglist1:
            if s.is_conflict:
              diff=True
              break
            elif s.is_hidden: # jeśli oba fragmenty są ukryte, to pokaż wersję superanotatora (jak konflikt)   
              diff=True
              break
        else:
          diff=True
              
        reflist = exc.segs.filter(user__isnull = True, xml_id__gte = exc_start, xml_id__lte = exc_end)
        seglist2.extend(reflist)
  #      print "seglist1: {}, seglist2: {}, reflist: {}, start: {}, end: {}".format(seglist1, seglist2, reflist, exc_start, exc_end)
       
        for seg in seglist1:
          seg.ref_id = ref
          if not diff:
            seg.ref_rejected = Reject_status.NO
          seg.save()
          
        for seg in seglist2:
          seg.ref_id = ref
          if not diff:
            seg.ref_rejected = Reject_status.YES
          seg.save()
          
        i = exc_end + 1
        ref += 1
        
      # jeśli zostały jakieś segmenty u superanotatora, to powinny być dołączone do ostatniego ref_id (czyli ref - 1)
      ref -= 1
      segs = exc.segs.filter(xml_id__gt = j, user__isnull = True, ref_id = 0)
      for seg in segs:
        seg.ref_id = ref
        if not diff:
          seg.ref_rejected = Reject_status.YES
        seg.save()
    
 #   print "Ustawianie danych dla superanotacji: {}".format(time.clock() - t)
    
  segs = exc.segs.exclude(ref_id = 0).order_by('ref_id', 'user', 'xml_id', 'choice_id', 'paren_id')
#  wd = Workdetail.objects.get(excerpt = exc, user = request.user)
#  prv, nxt = getPrevNextExc(exc, request.user)
  
#  exc_st = exc.details.filter(user = request.user)[0].work_status
#  if exc_st in [Work_status.ASSIGNED, Work_status.REASSIGNED]:
#    can_edit = 1
# else:
#   can_edit = 0
  
  context = {
    'exc': exc,
    'can_edit': 1,
    'mode': 'referee',
#    'next_exc': nxt,
#    'prev_exc': prv,
    'segs': segs,
#    'notes': wd.notes
  }

  return render(request, 'annotation/superexcerpt.html', context)

@login_required
def report_full(request):
#  annot_list = Workdetail.objects.filter(pay_date__isnull=False).order_by('pay_date', 'user__username', 'excerpt__text__text_name', 'excerpt__xml_id')
#  ref_list = Workdetail.objects.filter(ref_user__isnull=False, ref_pay_date__isnull=False).order_by('ref_pay_date', 'ref_user__username', 'excerpt__text__text_name', 'excerpt__xml_id').values('ref_pay_date', 'ref_user__username', 'ref_user__first_name', 'ref_user__last_name', 'excerpt__text__text_name', 'excerpt__xml_id').distinct()

  annot_qs = Workdetail.objects.filter(pay_date__isnull=False)
  annot_dates = annot_qs.order_by('-pay_date').values_list('pay_date', flat=True).distinct()

  ref_qs = Workdetail.objects.filter(ref_user__isnull=False, ref_pay_date__isnull=False)
  ref_dates = ref_qs.order_by('-ref_pay_date').values_list('ref_pay_date', flat=True).distinct()

  annot_list = []
  ref_list = []
 
  for d in annot_dates:
    annot_list.append({'d': d, 'l': annot_qs.filter(pay_date=d).order_by('pay_date', 'user__username', 'excerpt__text__text_name', 'excerpt__xml_id').values('pay_date', 'user__username', 'user__first_name', 'user__last_name', 'excerpt__text__text_name', 'excerpt__xml_id', 'pay_account__username')})

  for d in ref_dates:
    ref_list.append({'d': d, 'l': ref_qs.filter(ref_pay_date=d).order_by('ref_pay_date', 'ref_user__username', 'excerpt__text__text_name', 'excerpt__xml_id').values('ref_pay_date', 'ref_user__username', 'ref_user__first_name', 'ref_user__last_name', 'excerpt__text__text_name', 'excerpt__xml_id', 'ref_pay_account__username').distinct()})

  context = {
    'annot_list': annot_list,
    'ref_list': ref_list
  }
  return render(request, 'annotation/report_full.html', context)
  
@login_required
def report(request):
  annot_qs = Workdetail.objects.filter(pay_date__isnull=True, work_status__gt = 3)
  annot_users = annot_qs.values_list('user', flat=True).distinct()
  ref_qs = Workdetail.objects.filter(ref_user__isnull = False, ref_pay_date__isnull = True, work_status__gt = 5)
  ref_users = ref_qs.values_list('ref_user', flat=True).distinct()

  annot_list = []
  ref_list = []
 
  for u_id in annot_users:
    u = User.objects.get(pk=u_id)
    id_list = [elem.pk for elem in annot_qs.filter(user=u)]
    annot_list.append({'u': u, 'id_list': id_list, 'l': annot_qs.filter(user=u).order_by('user__username', 'excerpt__text__text_name', 'excerpt__xml_id').values('user', 'user__username', 'user__first_name', 'user__last_name', 'excerpt__text__text_name', 'excerpt__xml_id')})

  for u_id in ref_users:
    u = User.objects.get(pk=u_id)
    id_list = [elem.pk for elem in ref_qs.filter(ref_user=u)]
    ref_list.append({'u': u, 'id_list': id_list, 'l': ref_qs.filter(ref_user=u).order_by('ref_user__username', 'excerpt__text__text_name', 'excerpt__xml_id').values('ref_user', 'ref_user__username', 'ref_user__first_name', 'ref_user__last_name', 'excerpt__text__text_name', 'excerpt__xml_id').distinct()})

  if len(annot_list):
    annot_list = sorted(annot_list, key=lambda(k): len(k['l']), reverse=True)
  if len(ref_list):
    ref_list = sorted(ref_list, key=lambda(k): len(k['l']), reverse=True)

  context = {
    'annot_list': annot_list,
    'ref_list': ref_list,
  }
  return render(request, 'annotation/report.html', context)
  
@transaction.atomic
def report_post(request):
  dataJSON = json.loads(request.body)
  action = dataJSON.get('action', None)
  resp = json.dumps('OK')
  try:
    w_qs = Workdetail.objects.select_for_update().filter(pk__in=dataJSON['ids'])
    if action == 'annotPay':
      if len(w_qs.filter(pay_date__isnull = False)) > 0:
        resp = json.dumps(u'Próbki zostały w międzyczasie rozliczone.')
      else:
        for workdet in w_qs:
          workdet.pay_date = date.today()
          workdet.pay_account = request.user
          workdet.save()
    elif action == 'refPay':
      if len(w_qs.filter(ref_pay_date__isnull = False)) > 0:
        resp = json.dumps(u'Próbki zostały w międzyczasie rozliczone.')
      else:
        for workdet in w_qs:
          workdet.ref_pay_date = date.today()
          workdet.ref_pay_account = request.user
          workdet.save()
  except DatabaseError:
    json.dumps(u'Inna osoba właśnie zatwierdza te próbki.')
    
  return HttpResponse(resp)

def csv_export(request, exp_code):
  if exp_code == 'annot_full':
    annot_qs = Workdetail.objects.filter(pay_date__isnull=False).order_by('pay_date', 'user__username', 'excerpt__text__text_name', 'excerpt__xml_id').values('pay_date', 'user__username', 'user__first_name', 'user__last_name', 'excerpt__text__text_name', 'excerpt__xml_id', 'pay_account__username')

    response = HttpResponse(content_type='text/csv')
    response['Content-Disposition'] = 'attachment; filename="anotacja_raport.csv"'

    writer = csv.writer(response)
    response.write("\xEF\xBB\xBF")
    writer.writerow(['data rozliczenia', 'anotator', u'imię'.encode("utf-8"), 'nazwisko', 'tekst', u'numer próbki'.encode("utf-8"), u'rozliczający'.encode("utf-8")])
    for row in annot_qs:
      writer.writerow([row['pay_date'], row['user__username'], row['user__first_name'].encode("utf-8"), row['user__last_name'].encode("utf-8"), row['excerpt__text__text_name'].encode("utf-8"), row['excerpt__xml_id'], row['pay_account__username']])

  elif exp_code == 'ref_full':
    ref_qs = Workdetail.objects.filter(ref_user__isnull=False, ref_pay_date__isnull=False).order_by('ref_pay_date', 'ref_user__username', 'excerpt__text__text_name', 'excerpt__xml_id').values('ref_pay_date', 'ref_user__username', 'ref_user__first_name', 'ref_user__last_name', 'excerpt__text__text_name', 'excerpt__xml_id', 'ref_pay_account__username').distinct()

    response = HttpResponse(content_type='text/csv')
    response['Content-Disposition'] = 'attachment; filename="superanotacja_raport.csv"'

    writer = csv.writer(response)
    response.write("\xEF\xBB\xBF")
    writer.writerow(['data rozliczenia', 'superanotator', u'imię'.encode("utf-8"), 'nazwisko', 'tekst', u'numer próbki'.encode("utf-8"), u'rozliczający'.encode("utf-8")])
    for row in ref_qs:
      writer.writerow([row['ref_pay_date'], row['ref_user__username'], row['ref_user__first_name'].encode("utf-8"), row['ref_user__last_name'].encode("utf-8"), row['excerpt__text__text_name'].encode("utf-8"), row['excerpt__xml_id'], row['ref_pay_account__username']])

   
  return response
